/* SCCSID = "h/apmioctl.h, apm, r207, 8.231c 95/05/03" */
/****************************************************************************/
/*                                                                          */
/*                           IBM Confidential                               */
/*                                                                          */
/*                 Copyright (c) IBM Corporation 1993                       */
/*                           All Rights Reserved                            */
/*                                                                          */
/****************************************************************************/
/************************** START OF SPECIFICATIONS ***************************/
/*                                                                            */
/*   SOURCE FILE NAME:  APMIOCTL.H                                            */
/*                                                                            */
/*   DESCRIPTIVE NAME:  Constants, parameter and data packet definitions      */
/*                      for the generic IOCtl's provided by the APM driver.   */
/*                                                                            */
/*   FUNCTION: This module is the Advanced Power Management (APM) device      */
/*             driver generic IOCtl constants, parameter and data packet      */
/*             definitions include file.                                      */
/*                                                                            */
/*   NOTES:                                                                   */
/*      DEPENDENCIES: None                                                    */
/*      RESTRICTIONS: None                                                    */
/*                                                                            */
/*   ENTRY POINTS:  None                                                      */
/*                                                                            */
/*   EXTERNAL REFERENCES:  None                                               */
/*                                                                            */
/* Change Log                                                                 */
/*                                                                            */
/*  Mark    yy/mm/dd  Programmer      Comment                                 */
/*  ----    --------  ----------      -------                                 */
/*          92/10/01  Kip Harris      Original developer.                     */
/*          93/02/26  Frank Schroeder Added module and routine headers, fixed */
/*                                    Trap D on IOCtl pData and pParm with    */
/*                                    verifyaccess and aligned GIOERR_ return */
/*                                    codes with those specified in APM Spec. */
/*          93/03/25  Frank Schroeder Changed APMGIO_Category to be 12 decimal*/
/*                                    and not 12 hex.                         */
/*          95/04/21  Frank Schroeder Added APMGIO_BroadcastEvent IOCtl for   */
/*                                    broadcasting events without actually    */
/*                                    sending the request to the APM BIOS.    */
/*          95/05/02  Frank Schroeder Added APMGIO_RegDaemonThread IOCtl to   */
/*                                    broadcast events on dedicated thread    */
/*                                    rather than using context hook thread.  */
/*          95/05/02  Frank Schroeder Added APM daemon thread support.        */
/*          96/03/13  Dave Klein      Add APM 1.1 support.                    */
/*                                                                            */
/**************************** END OF SPECIFICATIONS ***************************/


/*---------------------------------------------------------------------------*
 * Category 12 (Power Management) IOCtl Function Codes                       *
 *---------------------------------------------------------------------------*/

#define APMGIO_Category           12    // Generic IOCtl Category for APM.

#define APMGIO_SendEvent        0x40    // Function Codes.
#define APMGIO_SetEventSem      0x41
#define APMGIO_ConfirmEvent     0x42    // 0x42 is UNDOCUMENTED.
#define APMGIO_BroadcastEvent   0x43    // 0x43 is UNDOCUMENTED.
#define APMGIO_RegDaemonThread  0x44    // 0x44 is UNDOCUMENTED.
#define APMGIO_OEMFunction      0x45
#define APMGIO_QueryStatus      0x60
#define APMGIO_QueryEvent       0x61
#define APMGIO_QueryInfo        0x62
#define APMGIO_QueryState       0x63

/*---------------------------------------------------------------------------*
 * Parameter and Data Packets passed in the Generic IOCTL Request Packet.    *
 *                                                                           *
 * Common Data Packet for All APM IOCtl's                                    *
 *---------------------------------------------------------------------------*/

typedef struct _APMGIO_DPKT {

  USHORT ReturnCode;

} APMGIO_DPKT, *NPAPMGIO_DPKT, FAR *PAPMGIO_DPKT;

// Error return codes.
#define GIOERR_PowerNoError             0
#define GIOERR_PowerBadSubId            1
#define GIOERR_PowerBadReserved         2
#define GIOERR_PowerBadDevId            3
#define GIOERR_PowerBadPwrState         4
#define GIOERR_PowerSemAlreadySetup     5
#define GIOERR_PowerBadFlags            6
#define GIOERR_PowerBadSemHandle        7
#define GIOERR_PowerBadLength           8
#define GIOERR_PowerDisabled            9
#define GIOERR_PowerNoEventQueue       10
#define GIOERR_PowerTooManyQueues      11
#define GIOERR_PowerBiosError          12
#define GIOERR_PowerBadSemaphore       13
#define GIOERR_PowerQueueOverflow      14
#define GIOERR_PowerStateChangeReject  15
#define GIOERR_PowerNotSupported       16
#define GIOERR_PowerDisengaged         17
#define GIOERR_PowerHighestErrCode     17


/*---------------------------------------------------------------------------*
 * Function 0x40, Send Power Event and Power Event Codes                     *
 *---------------------------------------------------------------------------*/

typedef struct _APMGIO_SENDEVENT_PPKT {         // Parameter Packet.

  USHORT SubId;         // Power Event Code.
  USHORT reserved;
  USHORT DevId;         // Used only for SetPowerState event.
  USHORT PwrState;      // Used only for SetPowerState event.

} APMGIO_SENDEVENT_PPKT, *NPAPMGIO_SENDEVENT_PPKT, FAR *PAPMGIO_SENDEVENT_PPKT;

// Power Event codes used in OS/2 messages for notification of Power Events.
// Reference APM Architecture Workbook.

#define APMEVENT_EventNumLow        0
#define APMEVENT_Idle               0
#define APMEVENT_WakeUp             1
#define APMEVENT_Busy               2
#define APMEVENT_EnableAPM          3
#define APMEVENT_DisableAPM         4
#define APMEVENT_BIOS_Defauts       5
#define APMEVENT_SetPowerState      6
#define APMEVENT_BatteryLow         7
#define APMEVENT_NormalResume       8
#define APMEVENT_CriticalResume     9
#define APMEVENT_StandbyResume     10
#define APMEVENT_EngageAPM         11
#define APMEVENT_DisengageAPM      12
#define APMEVENT_PowerStatus       13
#define APMEVENT_UpdateTime        14
#define APMEVENT_OemEvent          15
#define APMEVENT_QueryPowerState   16
#define APMEVENT_CriticalSuspend   17
#define APMEVENT_EventNumHigh      17


// Power State codes used in OS/2 messages for notification of Power Events.
// Reference APM Architecture Workbook.

#define APMSTATE_Ready          0
#define APMSTATE_Standby        1
#define APMSTATE_Suspend        2
#define APMSTATE_Off            3
#define APMSTATE_OEMfirst       0x0020
#define APMSTATE_OEMlast        0x007f

// Device IDs defined by the APM specification.
// These are the device class high order byte values.
// The low order byte is the unit number.

#define APMDEVID_Display        0x01
#define APMDEVID_DASD           0x02
#define APMDEVID_Parallel       0x03
#define APMDEVID_Serial         0x04
#define APMDEVID_Network        0x05
#define APMDEVID_PCMCIA         0x06

// Range of OEM defined device IDs
#define APMDEVID_OEMfirst       0xe000
#define APMDEVID_OEMlast        0xefff

/*---------------------------------------------------------------------------*
 * Function 0x41, Set Power Event Semaphore.                                 *
 *---------------------------------------------------------------------------*/

typedef ULONG HAPPSEM;  // Application semaphore handle.

typedef struct _APMGIO_SETEVENT_PPKT {          // Parameter Packet.

  USHORT  ReqFlags;     // Request flags.
  HAPPSEM hSem;         // Semaphore handle.
  ULONG   NotifyMask;   // Notification Mask.

} APMGIO_SETEVENT_PPKT, *NPAPMGIO_SETEVENT_PPKT, FAR *PAPMGIO_SETEVENT_PPKT;

/* Request flags bit definitions. */
#define  APM_SETEVENT_SETUP     0x0001    // Setup an Event Queue using
                                          // the semaphore submitted.
#define  APM_SETEVENT_RESET     0x0002    // Reset the semaphore.
#define  APM_SETEVENT_SEM16     0x0004    // 16-bit semaphore submitted.
#define  APM_SETEVENT_SEM32     0x0008    // 32-bit semaphore submitted.
#define  APM_SETEVENT_SEMNONE   0x0010    // No semaphore, update mask only.
#define  APM_SETEVENT_RESERVED  0xFFE0    // Reserved, future use.

#define  APM_SETEVENT_SEMTYPES  0x000C    // 16-bit and 32-bit semtypes only.
#define  APM_SETEVENT_SERVICES  0x0013    // Setup, reset, and semnone only.

/* Notification Mask Definitions. */
#define APMMASK_Idle              (0x00000001)
#define APMMASK_WakeUp            (0x00000002)
#define APMMASK_Busy              (0x00000004)
#define APMMASK_EnableAPM         (0x00000008)
#define APMMASK_DisableAPM        (0x00000010)
#define APMMASK_BIOS_Defaults     (0x00000020)
#define APMMASK_SetPowerState     (0x00000040)
#define APMMASK_BatteryLow        (0x00000080)
#define APMMASK_NormalResume      (0x00000100)
#define APMMASK_CriticalResume    (0x00000200)
#define APMMASK_StandbyResume     (0x00000400)
#define APMMASK_EngageAPM         (0x00000800)
#define APMMASK_DisengageAPM      (0x00001000)
#define APMMASK_PowerStatus       (0x00002000)
#define APMMASK_UpdateTime        (0x00004000)
#define APMMASK_OemEvent          (0x00008000)
#define APMMASK_QueryPowerState   (0x00010000)
#define APMMASK_AllAppNotifyBits  (0x0001fff8)

/* Mask of event notifications which have a Device ID parameter */
#define APMMASK_EventsUsingDevId  ( \
   APMMASK_EnableAPM | APMMASK_DisableAPM | APMMASK_SetPowerState | \
   APMMASK_EngageAPM | APMMASK_DisengageAPM | APMMASK_QueryPowerState )


/*---------------------------------------------------------------------------*
 * Function 0x42 (UNDOCUMENTED)  Confirm Power Event.                        *
 *---------------------------------------------------------------------------*/

typedef struct _APMGIO_CONFIRMEVENT_PPKT {      // Parameter Packet.

  USHORT SubId;         // Power Event Code.
  USHORT reserved;
  USHORT DevId;         // Used only for SetPowerState event.
  USHORT PwrState;      // Used only for SetPowerState event.
  USHORT Status;        // Indicates whether good or bad confirmation.

} APMGIO_CONFIRMEVENT_PPKT,
  *NPAPMGIO_CONFIRMEVENT_PPKT, FAR *PAPMGIO_CONFIRMEVENT_PPKT;

/* Legal values for Status member. */
#define APMCONFIRM_StatusProceedSuspend  0      // Complete the suspend action.
#define APMCONFIRM_StatusAbortSuspend    1      // Abort the suspend.


/*---------------------------------------------------------------------------*
 * Function 0x43, Broadcast Power Event                                      *
 *---------------------------------------------------------------------------*/

typedef struct _APMGIO_BROADCAST_PPKT {         // Parameter Packet.

  USHORT SubId;         // Power Event Code.
  USHORT reserved;
  USHORT DevId;         // Used only for SetPowerState event.
  USHORT PwrState;      // Used only for SetPowerState event.

} APMGIO_BROADCAST_PPKT, *NPAPMGIO_BROADCAST_PPKT, FAR *PAPMGIO_BROADCAST_PPKT;


/*---------------------------------------------------------------------------*
 * Function 0x44, Register Daemon Thread                                     *
 *---------------------------------------------------------------------------*/

typedef struct _APMGIO_REGDAEMON_PPKT {         // Parameter Packet.

  ULONG  reserved;

} APMGIO_REGDAEMON_PPKT, *NPAPMGIO_REGDAEMON_PPKT, FAR *PAPMGIO_REGDAEMON_PPKT;


/*---------------------------------------------------------------------------*
 * Function 0x45, OEM APM Function                                           *
 *---------------------------------------------------------------------------*/

/* Parameter Packet with general purpose registers.
 */
typedef struct _APMGIO_OEMFUNC_PPKT {

  USHORT ParmLength;    // Length, in bytes, of the Parm Packet.
  USHORT Flags;         // Output:  System flag word, low order byte.
  ULONG  EAX;           // Input/Output:  EAX register
  ULONG  EBX;           // Input/Output:  EBX register
  ULONG  ECX;           // Input/Output:  ECX register
  ULONG  EDX;           // Input/Output:  EDX register
  ULONG  ESI;           // Input/Output:  ESI register
  ULONG  EDI;           // Input/Output:  EDI register

} APMGIO_OEMFUNC_PPKT,
  *NPAPMGIO_OEMFUNC_PPKT, FAR *PAPMGIO_OEMFUNC_PPKT;

/* Extended Parameter Packet with added segment registers.
 */
typedef struct _APMGIO_OEMFUNCX_PPKT {

  USHORT ParmLength;    // Length, in bytes, of the Parm Packet.
  USHORT Flags;         // Output:  System flag word, low order byte.
  ULONG  EAX;           // Input/Output:  EAX register
  ULONG  EBX;           // Input/Output:  EBX register
  ULONG  ECX;           // Input/Output:  ECX register
  ULONG  EDX;           // Input/Output:  EDX register
  ULONG  ESI;           // Input/Output:  ESI register
  ULONG  EDI;           // Input/Output:  EDI register
  USHORT DS;            // Input/Output:  DS  register
  USHORT ES;            // Input/Output:  ES  register
  USHORT FS;            // Input/Output:  FS  register
  USHORT GS;            // Input/Output:  GS  register

} APMGIO_OEMFUNCX_PPKT,
  *NPAPMGIO_OEMFUNCX_PPKT, FAR *PAPMGIO_OEMFUNCX_PPKT;


/*---------------------------------------------------------------------------*
 * Function 0x60, Query Power Status                                         *
 * Reference MS/Intel APM specification for interpretation of status codes.  *
 *---------------------------------------------------------------------------*/

typedef struct _APMGIO_QSTATUS_PPKT {           // Parameter Packet.

  USHORT ParmLength;    // Length, in bytes, of the Parm Packet.
  USHORT Flags;         // Output:  Flags.
  UCHAR  ACStatus;      // Output:  AC line power status.
  UCHAR  BatteryStatus; // Output:  Battery power status
  UCHAR  BatteryLife;   // Output:  Battery power life
  UCHAR  BatteryTimeForm;  // Output: Form of battery time info
  USHORT BatteryTime;   // Output:  Remaining battery time.
  UCHAR  BatteryFlags;  // Output:  Battery status flags

} APMGIO_QSTATUS_PPKT, *NPAPMGIO_QSTATUS_PPKT, FAR *PAPMGIO_QSTATUS_PPKT;

typedef struct _APMGIO_10_QSTATUS_PPKT {        // Parameter Packet for
                                                // APM 1.0 interface
  USHORT ParmLength;    // Length, in bytes, of the Parm Packet.
  USHORT Flags;         // Output:  Flags.
  UCHAR  ACStatus;      // Output:  AC line power status.
  UCHAR  BatteryStatus; // Output:  Battery power status
  UCHAR  BatteryLife;   // Output:  Battery power status

} APMGIO_10_QSTATUS_PPKT;

/* Flags definition: */
#define  APMSTATUSFLAG_Enabled   ((USHORT) 0x01)  // Bit 0 is set if enabled.
#define  APMSTATUSFLAG_Engaged   ((USHORT) 0x02)  // Bit 1 is set if engaged.
#define  APMSTATUSFLAG_Connected ((USHORT) 0x04)  // Bit 2 is set if connection
                                                  //   to APM BIOS has been made.
/* Flags indicating full APM support */
#define  APMSTATUS_FULLSUPPORT  \
    (APMSTATUSFLAG_Enabled | APMSTATUSFLAG_Engaged | APMSTATUSFLAG_Connected)

/* BatteryTimeForm values: */
#define APMTIME_Seconds         0
#define APMTIME_Minutes         1
#define APMTIME_Unknown       255

/*---------------------------------------------------------------------------*
 * Function 0x61, Query Power Event                                          *
 *---------------------------------------------------------------------------*/

typedef struct _APMGIO_QEVENT_PPKT {           // Parameter Packet.

  USHORT  ParmLength;   // Length, in bytes, of the Parm Packet.
  USHORT  EventCount;   // Output:  number of events still queued
                        //          when this call returns.
  USHORT  Event;        // Output:  Power Event Code.
  USHORT  reserved;
  USHORT  DevId;        // Output:  Used only for SetPowerState event.
  USHORT  PwrState;     // Output:  Used only for SetPowerState event.

} APMGIO_QEVENT_PPKT, *NPAPMGIO_QEVENT_PPKT, FAR *PAPMGIO_QEVENT_PPKT;


/*---------------------------------------------------------------------------*
 * Function 0x62, Query Power Info                                           *
 * The values returned from this function correspond to the values returned  *
 * from the APM BIOS, function call AL=0 (Installation Check).               *
 *---------------------------------------------------------------------------*/

typedef struct _APMGIO_QINFO_PPKT {             // Parameter Packet.

  USHORT ParmLength;    // Length, in bytes, of the Parm Packet.
  USHORT BIOSFlags;     // Output:  APM BIOS flags as returned in CX reg.
  UCHAR  BIOSMajor;     // Output:  APM BIOS Major Version number (AH reg.)
  UCHAR  BIOSMinor;     // Output:  APM BIOS Minor Version number (AL reg.)
  UCHAR  DDMajor;       // Output:  Device driver version - major.
  UCHAR  DDMinor;       // Output:  Device driver version - minor.
  UCHAR  APMMajor;      // Output:  APM level being used  - major.
  UCHAR  APMMinor;      // Output:  APM level being used  - minor.

} APMGIO_QINFO_PPKT, *NPAPMGIO_QINFO_PPKT, FAR *PAPMGIO_QINFO_PPKT;

typedef struct _APMGIO_10_QINFO_PPKT {          // Parameter Packet for
                                                // APM 1.0 interface
  USHORT ParmLength;    // Length, in bytes, of the Parm Packet.
  USHORT BIOSFlags;     // Output:  APM BIOS flags as returned in CX reg.
  UCHAR  BIOSMajor;     // Output:  APM BIOS Major Version number (AH reg.)
  UCHAR  BIOSMinor;     // Output:  APM BIOS Minor Version number (AL reg.)
  UCHAR  DDMajor;       // Output:  Device driver version - major.
  UCHAR  DDMinor;       // Output:  Device driver version - minor.

} APMGIO_10_QINFO_PPKT;


/*---------------------------------------------------------------------------*
 * Function 0x63, Query Power State                                          *
 *---------------------------------------------------------------------------*/

typedef struct _APMGIO_QSTATE_PPKT {      // Parameter Packet.

  USHORT ParmLength;    // Length, in bytes, of the Parm Packet.
  USHORT DevId;         // Input:   Device ID.
  USHORT PwrState;      // Output:  Power state.

} APMGIO_QSTATE_PPKT,
  *NPAPMGIO_QSTATE_PPKT, FAR *PAPMGIO_QSTATE_PPKT;


/*---------------------------------------------------------------------------*
 *                                                                           *
 *               Inter Device Driver Communication Interface.                *
 *                                                                           *
 *---------------------------------------------------------------------------*/

/* APM IDC Function Codes. */
#define APMIDC_Register        0
#define APMIDC_Deregister      1
#define APMIDC_SendEvent       2
#define APMIDC_QueryStatus     3
#define APMIDC_QueryInfo       4
#define APMIDC_QueryDaemon     5
#define APMIDC_QueryState      6
#define APMIDC_OEMFunction     7

/* APM IDC Error Codes. */
#define APMIDC_InvalidFunction 128
#define APMIDC_InvalidHandle   129
#define APMIDC_TooManyClients  130
#define APMIDC_Busy            131

/*---------------------------------------------------------------------------*
 * IDC Packet Definitions.                                                   *
 *                                                                           *
 * Note that the first member of each packet must be a USHORT                *
 * which contains the function code which identifies the IDC function.       *
 *                                                                           *
 * All fields in these packets are inputs to the APM driver, unless          *
 * otherwise noted.                                                          *
 *---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*
 * IDC Function 0, Register as an APM Client                                 *
 *---------------------------------------------------------------------------*/

typedef struct _APMIDC_REGISTER_PKT {           // Registration Packet.

  USHORT  hClient;      // Output:  Client handle as assigned by APM driver.
  PFN     EventHandler; // Client's Event Handler routine.
  ULONG   NotifyMask;   // Notification Mask.
  SEL     ClientDS;     // Client's DS;  is restored prior to calling Handler.
  USHORT  DeviceId;     // Device conrtolled by this client.

} APMIDC_REGISTER_PKT, *NPAPMIDC_REGISTER_PKT, FAR *PAPMIDC_REGISTER_PKT;

/* When a registered event occurs, the Client driver's Handler
   is called with a pointer in ES:BX to the following packet
   (note semblance to an APMGIO_SENDEVENT packet):
 */

typedef struct _APMIDC_NOTIFY_PKT {             // Event notification packet.

  USHORT Function;      // Reserved, must be zero.
  USHORT SubId;         // Power Event Code.
  USHORT reserved;      // reserved, must be zero.
  USHORT DevId;         // Used only for SetPowerState event.
  USHORT PwrState;      // Used only for SetPowerState event.

} APMIDC_NOTIFY_PKT, *NPAPMIDC_NOTIFY_PKT, FAR *PAPMIDC_NOTIFY_PKT;


/*---------------------------------------------------------------------------*
 * IDC Function 1, Deregister Client                                         *
 *---------------------------------------------------------------------------*/

typedef struct _APMIDC_DEREGISTER_PKT {         // Deregistration Packet.

  USHORT  hClient;      // Client handle as assigned by APM driver.

} APMIDC_DEREGISTER_PKT, *NPAPMIDC_DEREGISTER_PKT, FAR *PAPMIDC_DEREGISTER_PKT;


/*---------------------------------------------------------------------------*
 * IDC Function 2, Send event.                                               *
 * This packet is identical to Cat 12 IOCtl 40 Send Event parameter packet.  *
 *---------------------------------------------------------------------------*/

typedef APMGIO_SENDEVENT_PPKT
  APMIDC_SENDEVENT_PKT, *NPAPMIDC_SENDEVENT_PKT, FAR *PAPMIDC_SENDEVENT_PKT;


/*---------------------------------------------------------------------------*
 * IDC Function 3, Query status.                                             *
 * This packet is identical to Cat 12 IOCtl 61 QueryStatus parameter packet. *
 *---------------------------------------------------------------------------*/

typedef APMGIO_QSTATUS_PPKT
  APMIDC_QSTATUS_PKT, *NPAPMIDC_QSTATUS_PKT, FAR *PAPMIDC_QSTATUS_PKT;


/*---------------------------------------------------------------------------*
 * IDC Function 4, Query Driver & Microcode Revision Levels                  *
 *                                                                           *
 * This packet is identical to Cat 12 IOCtl 62 QueryInfo parameter packet.   *
 *---------------------------------------------------------------------------*/

typedef APMGIO_QINFO_PPKT
  APMIDC_QINFO_PKT, *NPAPMIDC_QINFO_PKT, FAR *PAPMIDC_QINFO_PKT;


/*---------------------------------------------------------------------------*
 * IDC Function 6, Query Power State                                         *
 * This packet is identical to Cat 12 IOCtl 63 QueryState  parameter packet. *
 *---------------------------------------------------------------------------*/

typedef APMGIO_QSTATE_PPKT
  APMIDC_QSTATE_PKT, *NPAPMIDC_QSTATE_PKT, FAR *PAPMIDC_QSTATE_PKT;


/*---------------------------------------------------------------------------*
 * IDC Function 7, OEM APM Function                                          *
 * This packet is identical to Cat 12 IOCtl 45 OEM Function parameter packet.*
 *---------------------------------------------------------------------------*/

typedef APMGIO_OEMFUNC_PPKT
  APMIDC_OEMFUNC_PKT, *NPAPMIDC_OEMFUNC_PKT, FAR *PAPMIDC_OEMFUNC_PKT;


