/*      SCCSID = @(#)gas.h      1.0 95/02/01
        SCCSID = @(#)gas.h      1.0 95/02/01
  ****************************************************************************
  *                                                                          *
  *                 This file is IBM unique.                                 *
  *                 IBM Confidential                                         *
  *                 Copyright (c) IBM Corporation  1981, 1990                *
  *                 All Rights Reserved                                      *
  *                                                                          *
  ****************************************************************************

  @V111573 - Converted inc to h
  @PEN - added 32 bit log record declarations(LOGHEAD32) for CP20 DCR 1152

*/

//  logging facility devhlp call define

#define  LOGERROR    0x0001    // Log data in 16 bit format(LOGHEAD)
#define  LOGERROR32  0x0090    // Log data in 32 bit format(LOGHEAD32)

//  packet ids

#define  STDERROR    0x0001    //Packet id - OS/2 Standard Error
#define  SNAGENALERT 0x0002    //Packet id - SNA Generic Alert Subvector


//  Status Field Definitions

#define  PROCLOGNOTIFY  0x00000001 //Room for Proc Name, Log and Alert


//  logging facility structure for 16 bit log packets


typedef struct _LOGHEAD
{
   USHORT   NumPackets;    //Number of log packets dw 1
   USHORT   PacketLen;     //Length of log packet dw ?
   USHORT   PacketId;      //Packet id -  sna gen alert dw ?
   BYTE     Originator[8]; //Originator name db 'OS/2    '
   BYTE     Qualifier[4];  //Ascii selector db 'GA  '
   ULONG    Reserved;      //Reserved dd 0
   ULONG    Status;        //Status (Process Name, Log, Notify) dd PROCLOGNOTIFY
   ULONG    Time;          //Time field dd 0
   ULONG    Date;          //Date field dd 0
   BYTE     ProcessName[260];  //Process Name Field db 260 dup(0)
}  LOGHEAD;

#define DEFAULTLOGHEAD \
   {1,0,0,{'O','S','/','2',' ',' ',' ',' '},{'G','A',' ',' '},0,PROCLOGNOTIFY,0,0,{0}}


//  logging facility structure for 32 bit log packets(CP20 DCR 1152)

typedef struct _LOGHEAD32
{
   USHORT   PacketVer;     //Packet version number(1 is for 32 bit) dw 1
   USHORT   NumPackets;    //Number of log packets dw 1
   USHORT   PacketLen;     //Length of log packet dw ?
   USHORT   PacketId;      //Packet id -  sna gen alert dw ?
   ULONG    Status;        //Status (No Process Name) dd 0
   BYTE     Qualifier[4];  //Qualifier name db 'GA  '
   ULONG    Reserved;      //Reserved dd 0
   ULONG    Time;          //Time field dd 0
   ULONG    Date;          //Date field dd 0
   BYTE     Originator[8]; //Originator name db 'OS/2    '
   BYTE     FormatDLL[12]; //Formatting DLL name(none) db 12 dup(0)
}  LOGHEAD32;

#define DEFAULTLOGHEAD32 \
   {1,1,0,0,0,{'G','A',' ',' '},0,0,0,{'O','S','/','2',' ',' ',' ',' '},{0}}


//  generic alert subvectors


//  vector/subvector keys

#define  GAS_GEN_ALERT  0x92      //Generic Alert
#define  GAS_PROB_CAUSE 0x93      //Probable Causes
#define  GAS_USER_CAUSE 0x94      //User Caused
#define  GAS_INST_CAUSE 0x95      //Install Caused
#define  GAS_FAIL_CAUSE 0x96      //Failure Caused

//  alert types

#define  GAS_PERM_LOSS  0x01      //Permanent loss of availability
#define  GAS_TEMP_LOSS  0x02      //Temporary loss of avaibability
#define  GAS_PERF_LOSS  0x03      //Performance < acceptable level
#define  GAS_IMPENDING  0x11      //Impending problem
#define  GAS_UNKNOWN    0x12      //Unknown severity

//  alert description codes

#define  GAS_ADC_DEVOUTERROR     0x0012    //JT output device error
#define  GAS_ADC_PRINTER         0x0112    //Output device error (printer)
#define  GAS_ADC_DSKFAIL         0x1113    //I/O device error (disk failure)
#define  GAS_ADC_DSKOPER         0x1213    //I/O device error (operation error)
#define  GAS_ADC_DEVCOMERROR     0x0233    //JT unable to communicate with device
#define  GAS_ADC_DEVNOTREADY     0x1090    //JT device not ready
#define  GAS_ADC_PRTNOTREADY     0x1190    //operatore intervention rired (printer)
#define  GAS_ADC_PRTOUTOFPAPER   0x0292    //stock exhausted (printer)

//  probable causes description codes

#define  GAS_PCD_COMSTARTSTOP    0x0120    //JT Start/Stop Communications
#define  GAS_PCD_DEVOUTERROR     0x0062    //JT Output Device
#define  GAS_PCD_PRINTER         0x1062    //Output device (printer)
#define  GAS_PCD_DASD            0x1063    //I/O device (dasd device)

//  user causes subfields

#define  GAS_UC_USER    0x01      //User causes
#define  GAS_UC_RECACT  0x81      //Recommended action

//  user cause codes

#define  GAS_UCC_PRTPOWEROFF     0x0E02    //Printer powered off
#define  GAS_UCC_OUTOFPAPER      0x0353    //Printer out of paper
#define  GAS_UCC_PRTNOTREADY     0x1360    //Printer not ready

//  failure causes subfields

#define  GAS_FC_FAILURE    0x01      //Failure causes
#define  GAS_FC_RECACT     0x81      //Recommended action

//  failure cause codes

#define  GAS_FCC_DASD      0x0150    //DASD media
#define  GAS_FCC_DEVOUT    0x0062    //JT Output Device
#define  GAS_FCC_PRINTER   0x1062    //printer
#define  GAS_FCC_DISKDRV   0x1063    //Disk drive

//  recommended action codes

#define  GAS_RAC_ONLINEPD        0x0100    //Run online problem determination
#define  GAS_RAC_REF_GUIDE       0x0110    //Refer to operator's guide
#define  GAS_RAC_READY_DEVICE    0x0113    //Ready the device then retry
#define  GAS_RAC_ADD_PAPER       0x0616    //Add paper
#define  GAS_RAC_REVIEW_LOG      0x0122    //Review remote device logs
#define  GAS_RAC_CNTCT_SERV      0x0030    //Contact service representative


//  Description of Product Set ID subvector follows

#define  PSID_KEY       0x10             //Product Set ID Key
#define  SW_PID_KEY     0x11             //Software Product ID Key
#define  SWCLASS_IBM    0x04             //IBM Software Classification
#define  COMPID_KEY     0x02             //Component ID Subfield Key
#define  SW_CN_KEY      0x06             //Software Common Name key

#define  PSID_MAJOR     0xF2             //'2' Major version
#define  PSID_MINOR     0xF1             //'1' Minor version
#define  PSID_RELEASE   0xF0             //'0' Version release

//--------------------------------------------------------------------
//  Product Set ID subvector
//--------------------------------------------------------------------

typedef struct _PSIDS
{
   BYTE  PSIDS_Len;        //Product Set ID length         DB 52
   BYTE  PSIDS_Key;        //Product Set ID key            DB PSID_KEY
   BYTE  PSIDS_Retired;    //Retired field                 DB 0
   BYTE  PSIDS_SW_Len;     //Software Subvector length     DB 49
   BYTE  PSIDS_SW_Key;     //Software Subvector key        DB SW_PID_KEY
   BYTE  PSIDS_Class;      //Software class                DB SWCLASS_IBM
   BYTE  PSIDS_Comp_Len;   //Component ID Subfield length  DB 14
   BYTE  PSIDS_Comp_Key;   //Component ID Subfield key     DB COMPID_KEY

   BYTE  PSIDS_CompId[9];  //Componet ID db 0F5,0F6,0F2,0F1,0F0,0F7,0F7,0F0,0F1
                           //               '5','6','2','1','0','7','7','0','1'

   BYTE  PSIDS_CompRel[3]; //Release level, major version DB PSID_MAJOR == '2'
                           //minor version                DB PSID_MINOR == '1'
                           //version release              DB PSID_RELEASE == '0'

   BYTE  PSIDS_CN_Len;     //Software Common Name length  DB 32
   BYTE  PSIDS_SWCN_Key;   //Software Common Name key     DB SW_CN_KEY

   BYTE  PSIDS_SW_CN[30];  //'O'                          DB 0D6H
                           //'S'                          DB 0E2H
                           //'/'                          DB 061H
                           //'2'                          DB 0F2H
                           //' '                          DB 040H
                           //'B'                          DB 0C2H
                           //'A'                          DB 0C1H
                           //'S'                          DB 0E2H
                           //'E'                          DB 0C5H
                           //' '                          DB 040H
                           //'O'                          DB 0D6H
                           //'P'                          DB 0D7H
                           //'E'                          DB 0C5H
                           //'R'                          DB 0D9H
                           //'A'                          DB 0C1H
                           //'T'                          DB 0E3H
                           //'I'                          DB 0C9H
                           //'N'                          DB 0D5H
                           //'G'                          DB 0C7H
                           //' '                          DB 040H
                           //'S'                          DB 0E2H
                           //'Y'                          DB 0E8H
                           //'S'                          DB 0E2H
                           //'T'                          DB 0E3H
                           //'E'                          DB 0C5H
                           //'M'                          DB 0D4H
                           //' '                          DB 040H
                           //' '                          DB 040H
                           //' '                          DB 040H
                           //' '                          DB 040H
}  PSIDS;

#define DEFAULTPSIDS     \
{52,PSID_KEY,0,49,SW_PID_KEY,SWCLASS_IBM,14,COMPID_KEY,  \
 {0xf5,0xf6,0xf2,0xf1,0xf0,0xf7,0xf7,0xf0,0xf1},         \
 {PSID_MAJOR,PSID_MINOR,PSID_RELEASE},32,SW_CN_KEY,                             \
 {0xD6,0xE2,0x61,0xF2,0x40,                              \
  0xC2,0xC1,0xE2,0xC5,0x40,                              \
  0xD6,0xD7,0xC5,0xD9,0xC1,0xE3,0xC9,0xD5,0xC7,0x40,     \
  0xE2,0xE8,0xE2,0xE3,0xC5,0xD4,0x40,0x40,0x40,0x40}}

//--------------------------------------------------------------------
//  generic alert subvector
//--------------------------------------------------------------------
typedef struct _GAS
{
   BYTE     GAS_Len;       //length of G.A. Subvector             DB 0BH
   BYTE     GAS_Key;       //Generic Alert                        DB GAS_GEN_ALERT
   USHORT   GAS_Flags;     //Flags = !initiated, !held, !delayed  DW 0
   BYTE     GAS_Type;      //Alert type                           DB ?
   USHORT   GAS_Desc;      //Alert description                    DW ?
   ULONG    GAS_AlertId;   //Alert ID number                      DD ?
}  GAS;

#define DEFAULTGAS {0x0b,GAS_GEN_ALERT,0,0,0,0}

//--------------------------------------------------------------------
//  probable cause subvector
//--------------------------------------------------------------------
typedef struct _GAS_PCS
{
   BYTE     GAS_PCS_Len;   //length of P.C. Subvector  DB 4
   BYTE     GAS_PCS_Key;   //Probable causes           DB GAS_PROB_CAUSE
   USHORT   GAS_PCS_Desc;  //Description code          DW ?
}  GAS_PCS;

#define DEFAULTGASPCS {4,GAS_PROB_CAUSE,0}

//--------------------------------------------------------------------
//  user cause subvector
//--------------------------------------------------------------------
typedef struct _GAS_UCS
{
   BYTE     GAS_UCS_Len;      //length of U.C. Subvector DB 0AH
   BYTE     GAS_UCS_Key;      //User causes              DB GAS_USER_CAUSE
   BYTE     GAS_UCS_SFLen1;   //length of U.C. Subvector DB 4
   BYTE     GAS_UCS_SFKey1;   //User causes              DB GAS_UC_USER
   USHORT   GAS_UCS_CauseCd;  //Cause code               DW ?
   BYTE     GAS_UCS_SFLen2;   //length of R.A. Subvector DB 4
   BYTE     GAS_UCS_SFKey2;   //Recommended actions      DB GAS_UC_RECACT
   USHORT   GAS_UCS_RecAct;   //Recommended action code  DW ?
}  GAS_UCS;

#define DEFAULTGASUCS {0x0a,GAS_USER_CAUSE,4,GAS_UC_USER,0,4,GAS_UC_RECACT,0}

//--------------------------------------------------------------------
//  failure causes subvector
//--------------------------------------------------------------------
typedef struct _GAS_FCS
{
   BYTE     GAS_FCS_Len;      //length of F.C. Subvector DB 0AH
   BYTE     GAS_FCS_Key;      //Failure causes           DB GAS_FAIL_CAUSE
   BYTE     GAS_FCS_SFLen1;   //length of F.C. Subvector DB 4
   BYTE     GAS_FCS_SFKey1;   //Failure causes           DB GAS_FC_FAILURE
   USHORT   GAS_FCS_CauseCd;  //Cause code               DW ?
   BYTE     GAS_FCS_SFLen2;   //length of R.A. Subvector DB 4
   BYTE     GAS_FCS_SFKey2;   //Recommended actions      DB GAS_FC_RECACT
   USHORT   GAS_FCS_RecAct;   //Recommended action code  DW ?
}  GAS_FCS;

#define DEFAULTGASFCS \
{0x0a,GAS_FAIL_CAUSE,4,GAS_FC_FAILURE,0,4,GAS_FC_RECACT,0}
