;**************************************************************************
;*
;* SOURCE FILE NAME =  ABIOS.H
;*
;* DESCRIPTIVE NAME =  ADD/DM include file
;*                     ABIOS Structures/Defines for Disk and Diskette
;*                     and SCB Devices
;*
;* Copyright : COPYRIGHT IBM CORPORATION, 1991, 1992
;*             LICENSED MATERIAL - PROGRAM PROPERTY OF IBM
;*             REFER TO COPYRIGHT INSTRUCTION FORM#G120-2083
;*             RESTRICTED MATERIALS OF IBM
;*             IBM CONFIDENTIAL
;*
;* VERSION = V2.0
;*
;* DATE
;*
;* DESCRIPTION :
;*
;* Purpose:
;*
;*
;*
;*
;* FUNCTIONS  :
;*
;*
;*
;* NOTES
;*
;*
;* STRUCTURES
;*
;* EXTERNAL REFERENCES
;*
;*
;*
;* EXTERNAL FUNCTIONS
;*
;* CHANGE ACTIVITY =
;*   DATE      FLAG        APAR   CHANGE DESCRIPTION
;*   --------  ----------  -----  --------------------------------------
;*
;*   08/04/92      MFA     51531  Removable media control
;*
;***************************************************************************  
; 
;* ABIOS Entry Point Codes
ABIOS_EP_START	EQU	00H 
ABIOS_EP_INTERRUPT	EQU	01H 
ABIOS_EP_TIMEOUT	EQU	02H 
; 
;* ABIOS Function Codes
;  ABIOS Function Codes Common to all devices   
ABFC_DEFAULT_INT_HANDLER	EQU	00H 
ABFC_RET_LID_PARMS	EQU	01H 
ABFC_READ_DEVICE_PARMS	EQU	03H 
ABFC_SET_DEVICE_PARMS	EQU	04H 
ABFC_RESET_DEVICE	EQU	05H 
;  ABIOS Function Codes Specific to Diskette - Device ID 0x01   
ABFC_DSKT_RESET_INTERRUPT	EQU	07H 
ABFC_DSKT_READ	EQU	08H 
ABFC_DSKT_WRITE	EQU	09H 
ABFC_DSKT_FORMAT	EQU	0AH 
ABFC_DSKT_VERIFY	EQU	0BH 
ABFC_DSKT_READ_MEDIA_PARMS	EQU	0CH 
ABFC_DSKT_SET_MEDIA_TYPE	EQU	0DH 
ABFC_DSKT_READ_CHGSIGNAL	EQU	0EH 
ABFC_DSKT_TURN_OFF_MOTOR	EQU	0FH 
ABFC_DSKT_INT_STATUS	EQU	10H 
ABFC_DSKT_GET_MEDIA_TYPE	EQU	11H 
ABFC_DSKT_LOCK	EQU	12H                      ;  @V51531   
ABFC_DSKT_UNLOCK	EQU	13H                      ;  @V51531   
ABFC_DSKT_EJECT	EQU	14H                      ;  @V51531   
ABFC_DSKT_STATUS	EQU	16H                      ;  @V51531   
;  ABIOS Function Codes Specific to Fixed Disk - Device ID 0x02   
ABFC_DISK_READ	EQU	08H 
ABFC_DISK_WRITE	EQU	09H 
ABFC_DISK_WRITE_VERIFY	EQU	0AH 
ABFC_DISK_VERIFY	EQU	0BH 
ABFC_DISK_INTERRUPT_STATUS	EQU	0CH 
ABFC_DISK_SET_DMA_PACING	EQU	10H 
ABFC_DISK_RET_DMA_PACING	EQU	11H 
ABFC_DISK_TRANSFER_SCB	EQU	12H 
ABFC_DISK_DEALLOC_LID	EQU	14H 
ABFC_DISK_GET_SCSI_PARMS	EQU	1AH 
;  ABIOS Function Codes Specific to the SCSI Adapter - Device ID 0x17   
ABFC_SCSIA_RET_DEVICE_CONFIG	EQU	0BH 
ABFC_SCSIA_RET_INTERRUPT_LID	EQU	0CH 
ABFC_SCSIA_ENABLE_CACHE	EQU	0DH 
ABFC_SCSIA_DISABLE_CACHE	EQU	0EH 
ABFC_SCSIA_CACHE_STATUS	EQU	0FH 
ABFC_SCSIA_SET_DMA_PACING	EQU	10H 
ABFC_SCSIA_RET_DMA_PACING	EQU	11H 
ABFC_SCSIA_TRANSFER_SCB	EQU	12H 
;  ABIOS Function Codes Specific to the SCSI Peripheral Type - Device ID 0x18   
ABFC_SCSIP_SET_DEV_TIMEOUT	EQU	10H 
ABFC_SCSIP_READ_DEV_TIMEOUT	EQU	11H 
ABFC_SCSIP_TRANSFER_SCB	EQU	12H 
ABFC_SCSIP_DEALLOC_SCSI_DEV	EQU	14H 
ABFC_SCSIP_ALLOC_SCSI_DEV	EQU	15H 
ABFC_SCSIP_RET_TYPE_COUNT	EQU	16H 
ABFC_SCSIP_ABORT	EQU	17H 
; 
;*
;*   ABIOS REQUEST BLOCK STRUCTURES
;*
; 
;*  ABIOS Request Block Header

ABRBH	STRUC
ABH_Length	DW	?                 ;  00H -  Request Block Length    
ABH_LID	DW	?                    ;  02H -  Logical ID              
ABH_Unit	DW	?                   ;  04H -  Unit                    
ABH_Function	DW	?               ;  06H -  Function                
ABH_Reserved_1	DD	?             ;  08H -  Reserved                
ABH_RC	DW	?                     ;  0CH -  Return Code             
ABH_Timeout	DW	?                ;  0EH -  Time-Out                
ABRBH	ENDS
; 
;* Return LID Parms             - 0x01

ABRB_RETLIDPARMS	STRUC
ABRLP_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header       
ABRLP_HwIntLevel	DB	?             ;  10H - Hardware Interrupt Level   
ABRLP_ArbLevel	DB	?               ;  11H - Arbitration Level          
ABRLP_DeviceID	DW	?               ;  12H - Device ID                  
ABRLP_cUnits	DW	?                 ;  14H - Count of Units             
ABRLP_LIDFlags	DW	?               ;  16H - Logical ID Flags           
ABRLP_RBLength	DW	?               ;  18H - Request Block Length       
ABRLP_SecDeviceID	DB	?            ;  1AH - Secondary Device ID        
ABRLP_Revision	DB	?               ;  1BH - Revision                   
ABRLP_Reserved_1	DD	?             ;  1CH - Reserved, MBZ              
ABRB_RETLIDPARMS	ENDS
LF_16MB_SUPPORT	EQU	0020H 
LF_CONCURRENT	EQU	0008H 
LF_PHYSICAL_PTRS	EQU	0002H 
LF_LOGICAL_PTRS	EQU	0001H 
; 
;* Diagnostics

ABRBDIAGS	STRUC
ABD_Reserved_5	DB	?             ;  Reserved                       
ABD_ErrorLogLength	DB	?         ;  Error log length               
ABD_IntLevel	DB	?               ;  Interrupt level                
ABD_ArbLevel	DB	?               ;  Arbitration level              
ABD_DeviceID	DW	?               ;  Device ID                      
ABD_CmdtoCtrlr	DB	?             ;  Command to controller          
ABD_St0	DB	?                    ;                                 
ABD_St1	DB	?                    ;                                 
ABD_St2	DB	?                    ;                                 
ABD_Cylinder	DB	?               ;                                 
ABD_Head	DB	?                   ;                                 
ABD_Sector	DB	?                 ;                                 
ABD_SectorSize	DB	?             ;                                 
ABRBDIAGS	ENDS
; 
;* Read Device Parms (Diskette)   0x03

ABRB_DSKT_READDEVPARMS	STRUC
AB1RDP_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header       
AB1RDP_SectorsPerTrack	DW	?        ;  10H - Sectors per track          
AB1RDP_BlockSize	DW	?              ;  12H - Size of sector in bytes    
AB1RDP_DevCtrlFlags	DW	?           ;  14H - Device Control Flags       
AB1RDP_UnitType	DW	?               ;  16H - Diskette Unit Type         
AB1RDP_Reserved_1	DW	?             ;  18H - Reserved, MBZ              
AB1RDP_Undefined_1	DW	?            ;  1AH - Undefined                  
AB1RDP_MotorOffTime	DD	?           ;  1CH - Motor off time (usecs)     
AB1RDP_MotorStartTime	DD	?         ;  20H - Motor start time (usecs)   
AB1RDP_Undefined_2	DW	?            ;  24H - Undefined                  
AB1RDP_cCylinders	DW	?             ;  26H - Max Cylinders for drive    
AB1RDP_Undefined_3	DW	?            ;  28H - Undefined                  
AB1RDP_cHeads	DB	?                 ;  2AH - Number of heads            
AB1RDP_RetryCount	DB	?             ;  2BH - Recommended retry count    
AB1RDP_FillByte	DB	?               ;  2CH - Fill byte for format       
AB1RDP_HeadSettleTime	DD	?         ;  2DH - Head Settle Time           
AB1RDP_ReadGap	DB	?                ;  31H - Gap length for RWV         
AB1RDP_FormatGap	DB	?              ;  32H - Gap length for format      
AB1RDP_DataLen	DB	?                ;  33H - Data Length                
ABRB_DSKT_READDEVPARMS	ENDS
;  BlockSize equates in ABRB_DSKT_READDEVPARMS & ABRB_DSKT_SETDEVPARMS   
DP_BLOCKSIZE_256	EQU	01H     ;  256 bytes per sector          
DP_BLOCKSIZE_512	EQU	02H     ;  512 bytes per sector          
;  DevCtrlFlag equates in ABRB_DSKT_READDEVPARMS                         
DP_ABDEFGAPLEN	EQU	0040H   ;  ABIOS defines FORMAT GAP      
DP_RECALREQUIRED	EQU	0008H   ;  Recalibrate is required       
DP_CONCURRENT_DSKT	EQU	0004H   ;  Current diskette unit         
DP_FORMATSUPPORTED	EQU	0002H   ;  Format Unit supported         
DP_CHANGELINE_AVAIL	EQU	0001H   ;  Change line avail on drive    
;  UnitType equates in ABRB_DSKT_READDEVPARMS                            
DP_DRIVENOTPRES	EQU	00H     ;  Drive not present             
DP_DRIVETYPE_360KB	EQU	01H     ;  360  KB diskette drive        
DP_DRIVETYPE_12OOKB	EQU	02H     ;  1.2  MB diskette drive        
DP_DRIVETYPE_720KB	EQU	03H     ;  720  KB diskette drive        
DP_DRIVETYPE_144OKB	EQU	04H     ;  1.44 MB diskette drive        
DP_DRIVETYPE_2880KB	EQU	06H     ;  2.88 MB diskette drive        
; 
;* Set Device Parms (Diskette)    0x04

ABRB_DSKT_SETDEVPARMS	STRUC
AB1SDP_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header       
AB1SDP_Reserved_1	DW	?             ;  10H - Reserved, MBZ              
AB1SDP_BlockSize	DW	?              ;  12H - Block Size                 
AB1SDP_Undefined_1	DB	29 DUP (?)        ;  14H - Undefined                  
AB1SDP_ReadGap	DB	?                ;  31H - Gap length for RWV         
AB1SDP_Undefined_2	DB	?            ;  32H - Undefined                  
AB1SDP_DataLen	DB	?                ;  33H - Data Length                
ABRB_DSKT_SETDEVPARMS	ENDS
; 
;* Reset/Initialize            (Diskette)     0x05

ABRB_DSKT_RESET	STRUC
AB1RS_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header        
AB1RS_Reserved_1	DW	?             ;  10H - Reserved, MBZ               
ABRB_DSKT_RESET	ENDS
; 
;* Disable Interrupt           (Diskette)     0x07

ABRB_DISABLE	STRUC
AB1DI_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header        
AB1DI_Undefined_1	DB	8 DUP (?)         ;  10H - Undefined                   
AB1DI_Reserved_1	DW	?             ;  18H - Reserved, MBZ               
ABRB_DISABLE	ENDS
; 
;* Read, Write, Verify (Diskette)    0x08,0x09,0x0B

ABRB_DSKT_RWV	STRUC
AB1RWV_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header        
AB1RWV_Reserved_1	DW	?             ;  10H - Reserved, MBZ               
AB1RWV_pIObuffer	DD	?              ;  12H - Logical ptr to I/O buffer   
AB1RWV_Reserved_2	DD	?             ;  16H - Reserved, MBZ               
AB1RWV_ppIObuffer	DD	?             ;  1AH - Physical ptr to I/O buffer  
AB1RWV_Reserved_3	DW	?             ;  1EH - Reserved, MBZ               
AB1RWV_WaitTime	DD	?               ;  20H - Wait time before resuming   
AB1RWV_cSectors	DW	?               ;  24H - Count of sectors to xfer    
AB1RWV_Cylinder	DW	?               ;  26H - Cylinder number (0-based)   
AB1RWV_Undefined_1	DW	?            ;  28H - Undefined                   
AB1RWV_Head	DB	?                   ;  2AH - Head number (0-based)       
AB1RWV_Undefined_2	DB	6 DUP (?)         ;  2BH - Undefined                   
AB1RWV_Sector	DW	?                 ;  31H - Sector number (1-based)     
AB1RWV_DiagS	DB	SIZE ABRBDIAGS DUP (?)                  ;  33H - Diagnostic status           
ABRB_DSKT_RWV	ENDS
; 
;* Format                        (Diskette)    0x0A

ABRB_DSKT_FORMAT	STRUC
AB1FMT_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header        
AB1FMT_Reserved_1	DW	?             ;  10H - Reserved, MBZ               
AB1FMT_pFormatTable	DD	?           ;  12H - Log ptr to format table     
AB1FMT_Reserved_2	DD	?             ;  16H - Reserved, MBZ               
AB1FMT_ppFormatTable	DD	?          ;  1AH - Phys ptr to format table    
AB1FMT_Reserved_3	DW	?             ;  1EH - Reserved, MBZ               
AB1FMT_WaitTime	DD	?               ;  20H - Wait time before resuming   
AB1FMT_Subfunction	DW	?            ;  24H - Subfunction number          
AB1FMT_Cylinder	DW	?               ;  26H - Cylinder number (0-based)   
AB1FMT_Undefined_1	DW	?            ;  28H - Undefined                   
AB1FMT_Head	DB	?                   ;  2AH - Head number (0-based)       
ABRB_DSKT_FORMAT	ENDS
; 
;* Read Media Parms (Diskette)    0x0C

ABRB_DSKT_READMEDIAPARMS	STRUC
AB1RMP_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header        
AB1RMP_SectorsPerTrack	DW	?        ;  10H - Sectors per track           
AB1RMP_BlockSize	DW	?              ;  12H - Size of sector in bytes     
AB1RMP_Undefined_1	DW	?            ;  14H - Undefined                   
AB1RMP_Reserved_1	DW	?             ;  16H - Reserved, MBZ               
AB1RMP_Undefined_2	DB	14 DUP (?)        ;  18H - Undefined                   
AB1RMP_cCylinders	DW	?             ;  26H - Number of cylinders         
AB1RMP_Undefined_3	DW	?            ;  28H - Undefined                   
AB1RMP_cHeads	DB	?                 ;  2AH - Number of heads             
AB1RMP_Undefined_4	DB	6 DUP (?)         ;  2BH - Undefined                   
AB1RMP_ReadGap	DB	?                ;  31H - Gap length for RWV          
AB1RMP_FormatGap	DB	?              ;  32H - Gap length for format       
AB1RMP_DataLen	DB	?                ;  33H - Data Length                 
ABRB_DSKT_READMEDIAPARMS	ENDS
; 
;* Set Media Type for Format (Diskette)   0x0D

ABRB_DSKT_SETMEDIATYPE	STRUC
AB1SMT_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header        
AB1SMT_SectorsPerTrack	DW	?        ;  10H - Sectors per track           
AB1SMT_BlockSize	DW	?              ;  12H - Size of sector in bytes     
AB1SMT_Undefined_1	DW	?            ;  14H - Undefined                   
AB1SMT_Reserved_1	DW	?             ;  16H - Reserved, MBZ               
AB1SMT_Undefined_2	DB	8 DUP (?)         ;  18H - Undefined                   
AB1SMT_WaitTime	DD	?               ;  20H - Wait time before resuming   
AB1SMT_Undefined_3	DW	?            ;  24H - Undefined                   
AB1SMT_cTracks	DW	?                ;  26H - Number of tracks to format  
AB1SMT_Undefined_4	DD	?            ;  28H - Undefined                   
AB1SMT_FillByte	DB	?               ;  2CH - Fill byte for format        
AB1SMT_Undefined_5	DB	5 DUP (?)         ;  2DH - Undefined                   
AB1SMT_FormatGap	DB	?              ;  32H - Gap length for format       
ABRB_DSKT_SETMEDIATYPE	ENDS
; 
;* Read Change Line Status   (Diskette)   0x0E

ABRB_DSKT_READCHGLINE	STRUC
AB1RCL_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header        
AB1RCL_ChangeLineStatus	DB	?       ;  10H - Change Line status          
AB1RCL_Undefined_1	DB	5 DUP (?)         ;  11H - Undefined                   
AB1RCL_Reserved_1	DW	?             ;  16H - Reserved, MBZ               
ABRB_DSKT_READCHGLINE	ENDS
;  ChangeLineStatus equates in ABRB_DSKT_REACHGLINE                         
CHANGELINE_INACTIVE	EQU	00H     ;  Change Line Inactive             
CHANGELINE_ACTIVE	EQU	06H     ;  Change Line Active               
; 
;* Turn Off Motor            (Diskette)   0x0F

ABRB_DSKT_MOTOROFF	STRUC
AB1MO_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header        
AB1MO_Undefined_1	DB	6 DUP (?)         ;  10H - Undefined                   
AB1MO_Reserved_1	DW	?             ;  16H - Reserved, MBZ               
ABRB_DSKT_MOTOROFF	ENDS
; 
;* Interrupt Status          (Diskette)   0x10

ABRB_DSKT_INTRSTATUS	STRUC
AB1IS_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header        
AB1IS_InterruptStatus	DB	?        ;  10H - Interrupt Status            
AB1IS_Undefined_1	DB	5 DUP (?)         ;  11H - Undefined                   
AB1IS_Reserved_1	DW	?             ;  16H - Reserved, MBZ               
ABRB_DSKT_INTRSTATUS	ENDS
;  InterruptStatus equates in ABRB_DSKT_INTRSTATUS                          
NO_INTERRUPT	EQU	00H     ;  No Interrupt                     
INTERRUPT_PENDING	EQU	01H     ;  Interrupt Pending                
; 
;* Get Media Type            (Diskette)   0x11

ABRB_DSKT_GETMEDIATYPE	STRUC
AB1GMT_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header        
AB1GMT_MediaType	DB	?              ;  10H - Diskette Media Type         
AB1GMT_Undefined_1	DB	5 DUP (?)         ;  11H - Undefined                   
AB1GMT_Reserved_1	DW	?             ;  16H - Reserved, MBZ               
ABRB_DSKT_GETMEDIATYPE	ENDS
;  MediaType equates in ABRB_DSKT_GETMEDIATYPE                              
MEDIA1MB	EQU	03H 
MEDIA2MB	EQU	04H 
MEDIA4MB	EQU	06H 
; 
;*                                                               @V51531
;* Drive control Functions        (Diskette)   0x12 0x13 0x14    @V51531
;*                                                               @V51531
AB_DRIVE_LOCKED	EQU	0001H                               ;  @V51531    

ABRB_DSKT_CONTROL	STRUC
AB1DKC_abrbh	DB	SIZE ABRBH DUP (?)            ;  00H - Request Block Header  @V51531   
AB1DKC_LockStatus	DB	?             ;  10H - Reserved        @V51531   
AB1DKC_LockCount	DB	?              ;  11H - Reserved        @V51531   
ABRB_DSKT_CONTROL	ENDS
; 
;* Read Device Parms (Fixed Disk) 0x03

ABRB_DISK_READDEVPARMS	STRUC
AB2RDP_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header        
AB2RDP_SectorsPerTrack	DW	?        ;  10H - Sectors per track           
AB2RDP_BlockSize	DW	?              ;  12H - Size of sector in bytes     
AB2RDP_DevCtrlFlags	DW	?           ;  14H - Device Control Flags        
AB2RDP_LUN	DB	?                    ;  16H - Logical Unit Number (SCSI)  
AB2RDP_Undefined_1	DB	?            ;  17H - Undefined                   
AB2RDP_cCylinders	DD	?             ;  18H - Count of cylinders          
AB2RDP_cHead	DB	?                  ;  1CH - Count of heads              
AB2RDP_RetryCount	DB	?             ;  1DH - Suggested retry count       
AB2RDP_Undefined_2	DW	?            ;  1EH - Undefined                   
AB2RDP_cRBA	DD	?                   ;  20H - Count of RBAs on this unit  
AB2RDP_Reserved_1	DD	?             ;  24H - Reserved                    
AB2RDP_Reserved_2	DW	?             ;  28H - Reserved                    
AB2RDP_Undefined_3	DW	?            ;  2AH - Undefined                   
AB2RDP_MaxXferCount	DW	?           ;  2CH - Max blocks per transfer     
ABRB_DISK_READDEVPARMS	ENDS
;  DevCtrlFlag equates in ABRB_DISK_READDEVPARMS                             
DP_SCBXFER	EQU	8000H   ;  SCB Transfer Function supported   
DP_SCSI_DEVICE	EQU	4000H   ;  Drive is SCSI device              
DP_FORMAT_UNIT	EQU	1000H   ;  Format Unit supported             
DP_FORMAT_TRACK	EQU	0800H   ;  Format Track supported            
DP_ST506	EQU	0400H   ;  Drive is ST506 device             
DP_CONCURRENT_DISK	EQU	0200H   ;  Concurrent disk unit per LID      
DP_EJECTABLE	EQU	0100H   ;  Ejectable unit                    
DP_MEDIA_ORGANIZATION	EQU	0080H   ;  0=Random, 1=Sequential            
DP_LOCKING_AVAIL	EQU	0040H   ;  Locking capability supported      
DP_READABLE	EQU	0020H   ;  Unit is readable                  
DP_CACHE_AVAIL	EQU	0010H   ;  Caching is supported              
DP_WRITE_FREQUENCY	EQU	0008H   ;  0=Write once, 1=Write many        
DP_CHGLINE_DISK	EQU	0004H   ;  Change line supported             
DP_POWER	EQU	0002H   ;  0=Power on, 1=Power off           
DP_LOGDATAPTR	EQU	0001H   ;  Device needs logical data poineter   
; 
;* Reset/Initialize               (Disk)      0x05

ABRB_DISK_RESET	STRUC
AB2RS_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header        
AB2RS_Reserved_1	DW	?             ;  10H - Reserved, MBZ               
AB2RS_Undefined_1	DB	22 DUP (?)        ;  12H - Undefined                   
AB2RS_WaitTime	DD	?               ;  28H - Wait time before resuming   
ABRB_DISK_RESET	ENDS
; 
;* Read, Write, WriteVerify, Verify (Fixed Disk)  0x08,0x09,0x0A,0x0B

ABRB_DISK_RWV	STRUC
AB2RWV_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header        
AB2RWV_Reserved_1	DW	?             ;  10H - Reserved, MBZ               
AB2RWV_pIObuffer	DD	?              ;  12H - Logical ptr to I/O buffer   
AB2RWV_Reserved_2	DW	?             ;  16H - Reserved, MBZ               
AB2RWV_Reserved_3	DW	?             ;  18H - Reserved, MBZ               
AB2RWV_ppIObuffer	DD	?             ;  1AH - Physical ptr to I/O buffer  
AB2RWV_Reserved_4	DW	?             ;  1EH - Reserved, MBZ               
AB2RWV_RBA	DD	?                    ;  20H - RBA                         
AB2RWV_Reserved_5	DD	?             ;  24H - Reserved, MBZ               
AB2RWV_WaitTime	DD	?               ;  28H - Wait time before resuming   
AB2RWV_cBlocks	DW	?                ;  2CH - Count of blocks to xfer     
AB2RWV_Flags	DB	?                  ;  2EH - Flags                       
AB2RWV_SoftError	DW	?              ;  2FH - Soft error word             
ABRB_DISK_RWV	ENDS
;  Flags equates in ABRB_DSKT_RWV                                           
RW_DONT_CACHE	EQU	01H     ;  Dont cache this request          
; 
;* Interrupt Status          (Disk)       0x0C

ABRB_DISK_INTRSTATUS	STRUC
AB2IS_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header        
AB2IS_InterruptStatus	DB	?        ;  10H - Interrupt Status            
AB2IS_Undefined_1	DB	5 DUP (?)         ;  11H - Undefined                   
AB2IS_Reserved_1	DW	?             ;  16H - Reserved, MBZ               
ABRB_DISK_INTRSTATUS	ENDS
;  InterruptStatus equates in ABRB_DISK_INTRSTATUS                          
NO_INTERRUPT	EQU	00H     ;  No Interrupt                     
INTERRUPT_PENDING	EQU	01H     ;  Interrupt Pending                
; 
;* Set DMA Pacing            (Disk)       0x10

ABRB_DISK_SETDMAPACING	STRUC
AB2SDMA_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header        
AB2SDMA_DMAPacing	DB	?              ;  10H - DMA Pacing value            
AB2SDMA_Undefined_1	DB	5 DUP (?)         ;  11H - Undefined                   
AB2SDMA_Reserved_1	DW	?             ;  16H - Reserved, MBZ               
AB2SDMA_Undefined_2	DB	16 DUP (?)        ;  18H - Undefined                   
AB2SDMA_WaitTime	DD	?               ;  28H - Wait Time before resuming   
ABRB_DISK_SETDMAPACING	ENDS
; 
;* Return DMA Pacing         (Disk)       0x11

ABRB_DISK_RETDMAPACING	STRUC
AB2RDMA_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header        
AB2RDMA_DMAPacing	DB	?              ;  10H - Current DMA pacing value    
AB2RDMA_Undefined_1	DB	5 DUP (?)         ;  11H - Undefined                   
AB2RDMA_Reserved_1	DW	?             ;  16H - Reserved, MBZ               
ABRB_DISK_RETDMAPACING	ENDS
ifndef INCL_NO_SCB
; 
;*  Transfer SCB Request Block  (Disk)  0x12

ABRB_DISK_XFERSCB	STRUC
AB2XSCB_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header         
AB2XSCB_ppSCB	DD	?                  ;  10H - physical pointer to SCB      
AB2XSCB_Reserved_1	DW	?             ;  14H - reserved, MBZ                
AB2XSCB_pSCBHdr	DB	SIZE PSCBHDR DUP (?)                ;  16H - logical pointer to SCB hdr   
AB2XSCB_Undefined_1	DW	?            ;  1AH - Undefined                    
AB2XSCB_Reserved_2	DW	?             ;  1CH - reserved, MBZ                
AB2XSCB_pLastSCBHdr	DB	SIZE PSCBHDR DUP (?)            ;  1EH - logical ptr to last SCB hdr  
AB2XSCB_Undefined_2	DD	?            ;  22H - Undefined                    
AB2XSCB_Reserved_3	DW	?             ;  26H - reserved, MBZ                
AB2XSCB_WaitTime	DD	?               ;  28H - Wait time before resuming    
AB2XSCB_Reserved_4	DW	?             ;  2CH - reserved, MBZ                
AB2XSCB_Flags	DB	?                  ;  2EH - bits 7-1 reserved            
                                        ;  bit 0: 0-normal length SCB         
                                        ;         1-long SCB                  
AB2XSCB_SoftError	DW	?              ;  2FH - Soft error word              
AB2XSCB_Undefined_3	DB	?            ;  31H - Undefined                    
AB2XSCB_Status	DB	?                 ;  32H - bits 7-1 reserved            
                                        ;  bit 0: 0-last SCB field not valid  
                                        ;         1-last SCB field valid      
ABRB_DISK_XFERSCB	ENDS
endif
; 
;*  Deallocate LID              (Disk)  0x14

ABRB_DISK_DEALLOCLID	STRUC
AB2DL_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header       
AB2DL_Undefined_1	DW	?            ;  10H - Undefined                  
AB2DL_SCSIdisknum	DW	?            ;  12H - SCSI disk number           
AB2DL_Undefined_2	DW	?            ;  14H - Undefined                  
AB2DL_Reserved_1	DW	?             ;  16H - Reserved, MBZ              
ABRB_DISK_DEALLOCLID	ENDS
; 
;*  Get SCSI Parms              (Disk)  0x1A

ABRB_DISK_GETSCSIPARMS	STRUC
AB2GSP_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header       
AB2GSP_Reserved_1	DW	?             ;  10H - Reserved                   
AB2GSP_PUN	DB	?                    ;  12H - SCSI PUN                   
AB2GSP_LUN	DB	?                    ;  13H - SCSI LUN                   
AB2GSP_LDN	DB	?                    ;  14H - SCSI LDN                   
AB2GSP_AdapterIndex	DB	?           ;  15H - SCSI Adapter Index         
AB2GSP_Port	DW	?                   ;  16H - SCSI base port address     
ABRB_DISK_GETSCSIPARMS	ENDS
; 
;* Reset/Initialize Adapter           (SCSIA) 0x05

ABRB_SCSIA_RESET_INIT	STRUC
AB17RSET_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header        
AB17RSET_ResetType	DB	?              ;  10H - Reset type                  
AB17RSET_Undefined_1	DB	5 DUP (?)         ;  11H - Undefined                   
AB17RSET_Reserved_1	DW	?             ;  16H - Reserved, MBZ               
AB17RSET_Undefined_2	DB	16 DUP (?)        ;  18H - Undefined fields            
AB17RSET_WaitTime	DD	?               ;  28H - Wait time before resuming   
AB17RSET_Undefined_3	DB	6 DUP (?)         ;  2CH - Undefined                   
AB17RSET_Status	DB	?                 ;  32H - Status                      
ABRB_SCSIA_RESET_INIT	ENDS
; 
;* Return Device Configuration Table  (SCSIA) 0x0B

ABRB_SCSIA_RETDEVICECFGTBL	STRUC
AB17RDCT_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header       
AB17RDCT_Reserved_1	DW	?             ;  10H - Reserved                   
AB17RDCT_pDevCfgTbl	DB	SIZE PBYTE DUP (?)             ;  12H - Logical ptr to table       
AB17RDCT_Reserved_2	DW	?             ;  16H - Reserved                   
ABRB_SCSIA_RETDEVICECFGTBL	ENDS
; 
;* Return Interrupting Locical ID      (SCSIA) 0x0C

ABRB_RETINTRLID	STRUC
AB17RIL_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header       
AB17RIL_IntStatus	DB	?              ;  10H - Interrupt status           
AB17RIL_Undefined_1	DB	?            ;  11H - Undefined                  
AB17RIL_IntLID	DW	?                 ;  12H - Interrupting LID           
AB17RIL_Undefined_2	DW	?            ;  14H - Undefined                  
AB17RIL_Reserved_1	DW	?             ;  16H - Reserved                   
ABRB_RETINTRLID	ENDS
; 
;* Use the same equates for IntStatus as those defined for Interrupt
;* status in the Return Interrupt Status Disk ABIOS RB.
; 
;* Read Device Parms                 (SCSIP)  0x03

ABRB_SCSIP_READDEVPARMS	STRUC
AB18RDP_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header       
AB18RDP_Reserved_1	DW	?             ;  10H - Reserved, MBZ              
AB18RDP_SCB_Level	DB	?              ;  12H - SCB compatibility level    
AB18RDP_AdapterIndex	DB	?           ;  13H - Adapter Index              
AB18RDP_DeviceFlags	DW	?            ;  14H - Device Flags               
AB18RDP_LUN	DB	?                    ;  16H - Logical Unit Number        
AB18RDP_PUN	DB	?                    ;  17H - Physical Unit Number       
AB18RDP_Undefined_1	DB	16 DUP (?)        ;  18H - Undefined fields           
AB18RDP_Reserved_2	DW	?             ;  28H - Reserved, MBZ              
ABRB_SCSIP_READDEVPARMS	ENDS
;  DeviceFlags equates in ABRB_SCSIP_READDEVPARMS                           
DEVDEFECTIVE	EQU	0001H     ;  device found defective by POST   
POWEROFF	EQU	0002H     ;  device is currently powered off  
CACHESUPPORT	EQU	0010H     ;  Adpater card cache supported     
; 
;* Reset/Initialize                  (SCSIP)  0x05

ABRB_SCSIP_RESET	STRUC
AB18RS_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header        
AB18RS_Undefined_1	DB	6 DUP (?)         ;  10H - Undefined                   
AB18RS_Reserved_1	DW	?             ;  16H - Reserved, MBZ               
AB18RS_Undefined_2	DB	16 DUP (?)        ;  18H - Undefined fields            
AB18RS_WaitTime	DD	?               ;  28H - Wait time before resuming   
AB18RS_Undefined_3	DB	6 DUP (?)         ;  2CH - Undefined                   
AB18RS_Status	DB	?                 ;  32H - Status                      
ABRB_SCSIP_RESET	ENDS
; 
;* Set Device Time-Out               (SCSIP)  0x10

ABRB_SCSIP_SETTIMEOUT	STRUC
AB18STO_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header        
AB18STO_Timeout	DB	?                ;  10H - Time-out value              
AB18STO_Reserved_1	DB	?             ;  11H - Reserved, MBZ               
AB18STO_Undefined_1	DB	4 DUP (?)         ;  12H - Undefined                   
AB18STO_Reserved_2	DW	?             ;  16H - Reserved, MBZ               
AB18STO_Undefined_2	DB	16 DUP (?)        ;  18H - Undefined fields            
AB18STO_WaitTime	DD	?               ;  28H - Wait time before resuming   
AB18STO_Undefined_3	DB	6 DUP (?)         ;  2CH - Undefined                   
AB18STO_Status	DB	?                 ;  32H - Status                      
ABRB_SCSIP_SETTIMEOUT	ENDS
; 
;* Read Device Time-Out              (SCSIP)  0x11

ABRB_SCSIP_READTIMEOUT	STRUC
AB18RTO_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header        
AB18RTO_Timeout	DB	?                ;  10H - Time-out value              
AB18RTO_Reserved_1	DB	?             ;  11H - Reserved, MBZ               
AB18RTO_Undefined_1	DB	4 DUP (?)         ;  12H - Undefined                   
AB18RTO_Reserved_2	DW	?             ;  16H - Reserved, MBZ               
ABRB_SCSIP_READTIMEOUT	ENDS
; 
;* Transfer SCB                      (SCSIP)  0x12

ABRB_SCSIP_TRANSFERSCB	STRUC
AB18TSCB_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header         
AB18TSCB_ppSCB	DD	?                  ;  10H - physical pointer to SCB      
AB18TSCB_Reserved_1	DW	?             ;  14H - reserved, MBZ                
AB18TSCB_pSCBHdr	DB	SIZE PSCBHDR DUP (?)                ;  16H - logical pointer to SCB hdr   
AB18TSCB_Undefined_1	DW	?            ;  1AH - Undefined                    
AB18TSCB_Reserved_2	DW	?             ;  1CH - reserved, MBZ                
AB18TSCB_Undefined_2	DB	8 DUP (?)         ;  1EH - Undefined                    
AB18TSCB_Reserved_3	DW	?             ;  26H - reserved, MBZ                
AB18TSCB_WaitTime	DD	?               ;  28H - Wait time before resuming    
AB18TSCB_Reserved_4	DW	?             ;  2CH - reserved, MBZ                
AB18TSCB_Flags	DB	?                  ;  2EH - bits 7-1 reserved            
                                        ;        bit 0: 0-normal length SCB   
                                        ;               1-long SCB            
AB18TSCB_Undefined_3	DB	3 DUP (?)         ;  2FH - Undefined                    
AB18TSCB_Status	DB	?                 ;  32H - bits 7-2 & 0 reserved        
                                        ;        bit 1: 0 - CCSB not reqrd    
                                        ;               1 - CCSB needed       
ABRB_SCSIP_TRANSFERSCB	ENDS
NORMAL_SCB	EQU	00H                ;  Flags - Normal length SCB          
LONG_SCB	EQU	01H                ;  Flags - Long SCB                   
TSB_NOT_REQD	EQU	00H             ;  Status - TSB is not needed         
TSB_NEEDED	EQU	02H             ;  Status - TSB is needed             
; 
;* Deallocate SCSI Peripheral Device (SCSIP)  0x14

ABRB_SCSIP_DEALLOC	STRUC
AB18DPD_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header        
AB18DPD_Undefined_1	DB	6 DUP (?)         ;  10H - Undefined                   
AB18DPD_Reserved_1	DW	?             ;  16H - Reserved, MBZ               
ABRB_SCSIP_DEALLOC	ENDS
; 
;* Allocate SCSI Peripheral Device   (SCSIP)  0x15

ABRB_SCSIP_ALLOC	STRUC
AB18APD_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header        
AB18APD_DevType	DB	?                ;  10H - Device peripheral type      
AB18APD_DevFlags	DB	?               ;  11H - Device type flags           
AB18APD_DevUnit	DW	?                ;  12H - nth device of this type     
AB18APD_Undefined_1	DW	?            ;  14H - Undefined                   
AB18APD_Reserved_1	DW	?             ;  16H - Reserved, MBZ               
ABRB_SCSIP_ALLOC	ENDS
;  DevFlags equates in RB_SCSIP_ALLOC                                     
DEVTYPE_CDROM	EQU	05H 
DEVTYPE_REMOVABLE	EQU	80H     ;  Device media is removable      
; 
;* Return Peripheral Type Count      (SCSIP)  0x16

ABRB_SCSIP_RETCOUNT	STRUC
AB18RC_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header        
AB18RC_DevType	DB	?                ;  10H - Device Peripheral type      
AB18RC_DevFlags	DB	?               ;  11H - Device type flags           
AB18RC_Undefined_1	DW	?            ;  12H - Undefined                   
AB18RC_DevCount	DW	?               ;  14H - Count of device type        
AB18RC_Reserved_1	DW	?             ;  16H - Reserved, MBZ               
ABRB_SCSIP_RETCOUNT	ENDS
; 
;* Abort                             (SCSIP)  0x17

ABRB_SCSIP_ABORT	STRUC
AB18AB_abrbh	DB	SIZE ABRBH DUP (?)                  ;  00H - Request Block Header        
AB18AB_Undefined_1	DB	6 DUP (?)         ;  10H - Undefined                   
AB18AB_Reserved_1	DW	?             ;  16H - Reserved, MBZ               
AB18AB_Undefined_2	DB	16 DUP (?)        ;  18H - Undefined fields            
AB18AB_WaitTime	DD	?               ;  28H - Wait time before resuming   
AB18AB_Undefined_3	DB	6 DUP (?)         ;  2CH - Undefined                   
AB18AB_Status	DB	?                 ;  32H - Status                      
ABRB_SCSIP_ABORT	ENDS
GENERIC_ABRB_SIZE	EQU	128 
; 
;* ABIOS Return Codes
;  ABIOS Return Codes Common to all Functions   
ABRC_COMPLETEOK	EQU	0000H 
ABRC_STAGEONINTERRUPT	EQU	0001H 
ABRC_STAGEONTIME	EQU	0002H 
ABRC_NOTMYINTERRUPT	EQU	0005H 
ABRC_ATTENTION	EQU	0009H 
ABRC_SPURIOUSINTERRUPT	EQU	0081H 
ABRC_BUSY	EQU	8000H 
ABRC_START	EQU	0FFFFH 
ABRC_UNSUPPORTED_LID	EQU	0C000H 
ABRC_UNSUPPORTED_FUNCTION	EQU	0C001H 
ABRC_UNSUPPORRTED_UNIT	EQU	0C003H 
ABRC_UNSUPPORTED_RB_LEN	EQU	0C004H 
ABRC_INVALID_PARM	EQU	0C005H 
ABRC_ERRORBIT	EQU	8000H 
ABRC_RETRYBIT	EQU	0100H 
ABRC_ERRORMSK	EQU	00FFH 
;  ABIOS Return Codes Specific to Diskette - Device ID 0x01     
ABRC_DSKT_WRITE_PROTECT	EQU	8003H 
ABRC_DSKT_MEDIA_CHANGED	EQU	8006H 
ABRC_DSKT_MEDIA_NOT_PRESENT	EQU	800DH 
ABRC_DSKT_NOCHGSIG	EQU	800EH 
ABRC_DSKT_INVALID_VALUE	EQU	800FH 
ABRC_DSKT_MEDIA_NOTSUPPORTED	EQU	8010H 
ABRC_DSKT_NO_MEDIA_SENSE	EQU	8011H 
ABRC_DSKT_RESET_FAIL	EQU	9009H 
ABRC_DSKT_ADDRMARK_NOTFND	EQU	9102H 
ABRC_DSKT_SECTOR_NOTFND	EQU	9104H 
ABRC_DSKT_DMA_IN_PROGRESS	EQU	9107H 
ABRC_DSKT_DMA_OVERRUN	EQU	9108H 
ABRC_DSKT_BAD_CRC	EQU	9110H 
ABRC_DSKT_BAD_CONTROLLER	EQU	9120H    ;  A120,B020                   
ABRC_DSKT_BAD_SEEK	EQU	9140H 
ABRC_DSKT_GENERAL_ERROR	EQU	9180H 
ABRC_DSKT_UNKNOWN_MEDIA	EQU	0C00CH 
;  ABIOS Return Codes Specific to Disk - Device ID 0x02     
ABRC_DISK_DEV_NOT_POWERED_ON	EQU	8001H    ;                              
ABRC_DISK_DEV_BLOCK_INIT_ERR	EQU	8002H    ;                              
ABRC_DISK_DEV_NOT_ALLOCATED	EQU	8003H    ;                              
ABRC_DISK_DMA_ARB_INVALID	EQU	800FH    ;                              
ABRC_DISK_BAD_COMMAND	EQU	9001H    ;  9101,A001,B001,B101         
ABRC_DISK_ADDRMARK_NOTFND	EQU	9002H    ;  9102,A002                   
ABRC_DISK_WRITE_PROTECT	EQU	9003H    ;  9103                        
ABRC_DISK_RECORD_NOTFND	EQU	9004H    ;  9104,A004                   
ABRC_DISK_RESET_FAIL	EQU	9005H    ;  9105,A005,A105              
ABRC_DISK_MEDIA_CHANGED	EQU	9006H    ;  9106                        
ABRC_DISK_CTRL_PARM_FAIL	EQU	9007H    ;  9107,A007,A107              
ABRC_DISK_DMA_FAIL	EQU	9008H    ;  9108                        
ABRC_DISK_DEFECTIVE_SECTOR	EQU	900AH    ;       A00A                   
ABRC_DISK_BAD_TRACK	EQU	900BH    ;       A00B                   
ABRC_DISK_FORMAT_ERROR	EQU	900DH    ;       A00D                   
ABRC_DISK_CAM_RV	EQU	900EH    ;       A00E                   
ABRC_DISK_CRC	EQU	9010H    ;       A010                   
ABRC_DISK_DEVICE_FAILED	EQU	9014H    ;  9114                        
ABRC_DISK_BUS_FAULT	EQU	9015H    ;  9115                        
ABRC_DISK_BAD_CONTROLLER	EQU	9020H    ;  9120,A020,A120,B020,B120    
ABRC_DISK_EQUIP_CHECK	EQU	9021H    ;  9121,A021,A121,B021,B121    
ABRC_DISK_BAD_SEEK	EQU	9040H    ;  9140,A040,A140              
ABRC_DISK_DEVICE_NORESPONSE	EQU	9080H    ;  9180,A080,A180,B080,B180    
ABRC_DISK_DRIVE_NOTREADY	EQU	90AAH    ;  91AA,A0AA,A1AA              
ABRC_DISK_UNDEFINED_ERROR	EQU	90BBH    ;  91BB,A0BB,A1BB,B0BB,B1BB    
ABRC_DISK_WRITE_FAULT	EQU	90CCH    ;  91CC,A0CC,A1CC              
ABRC_DISK_STATUS_ERROR	EQU	90E0H    ;  91E0                        
ABRC_DISK_INCOMPLETE_SENSE	EQU	90FFH    ;  91FF,A0FF,A1FF,B0FF,B1FF    
ABRC_DISK_ECC_CORRECTED	EQU	0A011H    ;                              
; 
;* Various diskette equates
;  Equates specific to NEC Diskette Controller                                
ABNEC	EQU	020Q      ;  Level of NEC Controller      
NEC_DSKT_READ	EQU	0E6H 
NEC_DSKT_WRITE	EQU	0C5H 
NEC_DSKT_FORMAT	EQU	4DH 
;  Other various diskette equates                                             
ABDISKETTELID	EQU	0001H 
HW_CHANGELINE_ACTIVE	EQU	80H 
ABCHGLINEACTIVE	EQU	00FCH 
; 
;* Various Fixed Disk Equates
ABRB_NOCACHE	EQU	01H   ;  ABRB_Flags bit 0, set=dont cache  
ABRETRYCOUNT	EQU	05H   ;  Device Error Retry Limit          
; 
;* ABIOS Device IDs
DEVID_INTERNAL	EQU	00H 
DEVID_DISKETTE	EQU	01H 
DEVID_DISK	EQU	02H 
DEVID_VIDEO	EQU	03H 
DEVID_KEYBOARD	EQU	04H 
DEVID_PARALLEL	EQU	05H 
DEVID_ASYNC	EQU	06H 
DEVID_SYSTEMTIMER	EQU	07H 
DEVID_RTC	EQU	08H 
DEVID_SYSTEMSVCS	EQU	09H 
DEVID_NMI	EQU	0AH 
DEVID_MOUSE	EQU	0BH 
DEVID_NVRAM	EQU	0EH 
DEVID_DMA	EQU	0FH 
DEVID_POS	EQU	10H 
DEVID_SECURITY	EQU	16H 
DEVID_SCSIADAPTER	EQU	17H 
DEVID_GENERICSCSI	EQU	18H 
