;**************************************************************************
;*
;* SOURCE FILE NAME =  ADDCALLS.H
;*
;* DESCRIPTIVE NAME =  C Function Prototypes/Structures for services
;*                     contained in ADDCALLS.LIB
;*                     This file also contains IN/OUT Port C-Macros.
;*
;* Copyright : COPYRIGHT IBM CORPORATION, 1991, 1992
;*             LICENSED MATERIAL - PROGRAM PROPERTY OF IBM
;*             REFER TO COPYRIGHT INSTRUCTION FORM#G120-2083
;*             RESTRICTED MATERIALS OF IBM
;*             IBM CONFIDENTIAL
;*
;* VERSION = V2.0
;*
;* DATE
;*
;* DESCRIPTION :
;*
;* Purpose:   ADDCALLS.LIB provides the following services:
;*
;*                 - C Callable Timer Manager
;*                 - Scatter/Gather List Emulation
;*                 - RBA to CHS conversion
;*                 - Diskette DMA channel setup
;*
;*
;* FUNCTIONS  :
;*
;*
;*
;* NOTES
;*
;*
;* STRUCTURES
;*
;* EXTERNAL REFERENCES
;*
;*
;*
;* EXTERNAL FUNCTIONS
;*
;* CHANGE ACTIVITY =
;*   DATE      FLAG        APAR   CHANGE DESCRIPTION
;*   --------  ----------  -----  --------------------------------------
;*   mm/dd/yy  @VR.MPPPXX  XXXXX  XXXXXXX
;*
;***************************************************************************  
; 
;* Xfer Buffer Data Structure

ADD_XFER_DATA	STRUC
ADDX_Mode	DW	?                         ;  Direction of xferdata        
ADDX_cSGList	DW	?                      ;  Count of S/G list elements   
ADDX_pSGList	DB	SIZE PSCATGATENTRY DUP (?)                      ;  Far pointer to S/G List      
ADDX_pBuffer	DB	SIZE PBYTE DUP (?)                      ;  Far pointer to buffer        
ADDX_numTotalBytes	DD	?                ;  Total bytes to copy          
ADDX_iSGList	DW	?                      ;  Current index of S/G List    
ADDX_SGOffset	DD	?                     ;  Current offset               
ADD_XFER_DATA	ENDS
SGLIST_TO_BUFFER	EQU	1                 ;  From S/G list to buffer      
BUFFER_TO_SGLIST	EQU	2                 ;  From buffer to S/G list      
; 
;* Xfer I/O Data Structure

ADD_XFER_IO	STRUC
ADDIO_Mode	DW	?                         ;  Direction of xferdata        
ADDIO_cSGList	DW	?                      ;  Count of S/G list elements   
ADDIO_pSGList	DB	SIZE PSCATGATENTRY DUP (?)                      ;  Far pointer to S/G List      
ADDIO_iPortAddress	DW	?                 ;  I/O Port Address             
ADDIO_numTotalBytes	DD	?                ;  Total bytes to copy          
ADDIO_iSGList	DW	?                      ;  Current index of S/G List    
ADDIO_SGOffset	DD	?                     ;  Current offset               
ADDIO_iSGListStart	DW	?                 ;  Save area - S/G List Index   
ADDIO_SGOffsetStart	DD	?                ;  Save area - S/G List Offset  
ADD_XFER_IO	ENDS
SGLIST_TO_PORT	EQU	1                 ;  From S/G list to I/O Port    
PORT_TO_SGLIST	EQU	2                 ;  From I/O Port to S/G list    
; 
;* Timer Data Structure

ADD_TIMER_DATA	STRUC
ADDT_Interval	DD	?                      ;  Interval value in millisecond   
ADDT_BackupInterval	DD	?                ;  Interval value for backup       
ADDT_NotifyEntry	DB	SIZE PFN DUP (?)                   ;  Notify address                  
ADDT_Parm_1	DB	SIZE PVOID DUP (?)                        ;  parameter which ADD wants       
ADDT_Parm_2	DB	SIZE PVOID DUP (?)                        ;  parameter which ADD wants       
ADD_TIMER_DATA	ENDS
; 
;* Timer Pool Structure

ADD_TIMER_POOL	STRUC
ADDT_MTick	DW	?                       ;  Milliseconds per timer tick     
ADDT_TimerData	DB	SIZE ADD_TIMER_DATA * 1 DUP (?)                ;  Interval value for backup       
ADD_TIMER_POOL	ENDS
; 
;* If the caller wants "n" timer elements, the size of data pool is
;*
;*    sizeof(ADD_TIMER_POOL) + (n-1)*(ADD_TIMER_DATA).
;*
; 
;* I/O Instruction macro
                                              ;  OUT                          
                                             ;  IN                           
                                             ;  OUTW                         
                                             ;  INW                          
                                             ;  OUTSW                         
                                             ;  INSW                          
; 
;* ADD Common Services
; 
;* ADD Common Services R / C
ADD_SUCCESS	EQU	0 
ADD_ERROR	EQU	1 
