;  SCCSID = %w% %e%   
;**************************************************************************
;*
;* SOURCE FILE NAME =  DHCALLS.H
;*
;* DESCRIPTIVE NAME =  ADD/DM include file
;*                     C Function Prototypes services in DHCALLS.LIB
;*
;* Copyright : COPYRIGHT IBM CORPORATION, 1991, 1992
;*             LICENSED MATERIAL - PROGRAM PROPERTY OF IBM
;*             REFER TO COPYRIGHT INSTRUCTION FORM#G120-2083
;*             RESTRICTED MATERIALS OF IBM
;*             IBM CONFIDENTIAL
;*
;* VERSION = V2.0
;*
;* DATE
;*
;* DESCRIPTION :
;*
;* Purpose:    DHCALLS.LIB provides a C-Callable interface to OS/2
;*             Device Help (DevHelp) services.
;*
;*             DevHelp services provide memory/interrupt/semaphore
;*             management services for ADDs and Device Managers.
;*
;*
;* FUNCTIONS  :
;*
;*
;*
;* NOTES
;*
;*
;* STRUCTURES
;*
;* EXTERNAL REFERENCES
;*
;*
;*
;* EXTERNAL FUNCTIONS
;*
;* CHANGE ACTIVITY =
;*   DATE      FLAG        APAR   CHANGE DESCRIPTION
;*   --------  ----------  -----  --------------------------------------
;*   10/22/93  @V74979     74979  Change DevHelp return codes from BOOL
;*                                to USHORT
;*
;*   10/22/93  @V74979     74979  Fix prototype DevHelp_DeRegister
;*                                              DevHelp_VMSetMem
;*                                              DevHelp_Beep
;*                                              DevHelp_RegisterBeep
;*
;*   10/22/93  @V74979     74979  Add DEVICECLASS_* equates for
;*                                DevHelp_RegisterDeviceClass.
;*
;*   12/02/93  @V76282     76282  Add defines for flags on various
;*                                DevHelps based on review of PDD Book
;*
;*   12/02/93  @V76282     76282  Fix prototype DevHelp_MonWrite
;*                                              DevHelp_SchedClock
;*
;*   12/02/93  @V76282     76282  Add prototype DevHelp_RegisterTmrDD
;*                                              DevHelp_DynamicAPI
;*
;*   05/31/94  @V85056     85056  Add DDTABLE type for DevHelp_AttachDD
;*
;*   02/06/95  @V111573   111573  Add prototype DevHelp_LogEntry
;*
;***************************************************************************  
; 
;* DevHelp Library Calls
; 
;* ABIOS Related
; 
;* Memory Management
;  DevHelp_AllocPhys   
MEMTYPE_ABOVE_1M	EQU	0 
MEMTYPE_BELOW_1M	EQU	1 
;  DevHelp_Lock   
LOCKTYPE_SHORT_ANYMEM	EQU	00H 
LOCKTYPE_LONG_ANYMEM	EQU	01H 
LOCKTYPE_LONG_HIGHMEM	EQU	03H 
LOCKTYPE_SHORT_VERIFY	EQU	04H 
;  DevHelp_PhysToUVirt   
SELTYPE_R3CODE	EQU	0 
SELTYPE_R3DATA	EQU	1 
SELTYPE_FREE	EQU	2 
SELTYPE_R2CODE	EQU	3 
SELTYPE_R2DATA	EQU	4 
SELTYPE_R3VIDEO	EQU	5 
;  DevHelp_VerifyAccess   
VERIFY_READONLY	EQU	0 
VERIFY_READWRITE	EQU	1 
; 
;* Request Packet Management
;  DevHelp_AllocRequestPacket   
WAIT_NOT_ALLOWED	EQU	0 
WAIT_IS_ALLOWED	EQU	1 
; 
;* Character Queue Management
;  DevHelp_QueueInit   

QUEUEHDR	STRUC
DHQH_QSize	DW	?
DHQH_QChrOut	DW	?
DHQH_QCount	DW	?
DHQH_Queue	DB	1 DUP (?)
QUEUEHDR	ENDS
; 
;* Inter-Device Driver Communications & Kernel Functions

IDCTABLE	STRUC
                                                                     ; @V85056  
DHID_Reserved	DW	3 DUP (?)                                         ; @V85056  
DHID_ProtIDCEntry	DD	?                           ; @V85056  
DHID_ProtIDC_DS	DW	?                                          ; @V85056  
                                                                     ; @V85056  
IDCTABLE	ENDS
                                                                     ; @V85056  
DHGETDOSV_SYSINFOSEG	EQU	1 
DHGETDOSV_LOCINFOSEG	EQU	2 
DHGETDOSV_VECTORSDF	EQU	4 
DHGETDOSV_VECTORREBOOT	EQU	5 
DHGETDOSV_YIELDFLAG	EQU	7                             ; @V76282  
DHGETDOSV_TCYIELDFLAG	EQU	8                             ; @V76282  
DHGETDOSV_DOSCODEPAGE	EQU	11                            ; @V76282  
DHGETDOSV_INTERRUPTLEV	EQU	13 
DHGETDOSV_DEVICECLASSTABLE	EQU	14                            ; @V76282  
DHGETDOSV_DMQSSELECTOR	EQU	15                            ; @V76282  
DHGETDOSV_APMINFO	EQU	16                            ; @V76282  

MSGTABLE	STRUC
DHMT_MsgId	DW	?                       ;  Message Id #                    
DHMT_cMsgStrings	DW	?                 ;  # of (%) substitution strings   
DHMT_MsgStrings	DB	SIZE PSZ * 1 DUP (?)               ;  Substitution string pointers    
MSGTABLE	ENDS
; 
;* Interrupt/Thread Management
;  DevHelp_RegisterStackUsage   

STACKUSAGEDATA	STRUC
DHRS_Size	DW	?
DHRS_Flags	DW	?
DHRS_IRQLevel	DW	?
DHRS_CLIStack	DW	?
DHRS_STIStack	DW	?
DHRS_EOIStack	DW	?
DHRS_NestingLevel	DW	?
STACKUSAGEDATA	ENDS
;  DevHelp_Block   
WAIT_IS_INTERRUPTABLE	EQU	0 
WAIT_IS_NOT_INTERRUPTABLE	EQU	1 
WAIT_INTERRUPTED	EQU	8003H 
WAIT_TIMED_OUT	EQU	8001H 
;  DevHelp_VideoPause   
VIDEO_PAUSE_OFF	EQU	0 
VIDEO_PAUSE_ON	EQU	1 
; 
;* Semaphore Management
;  DevHelp_SemHandle   
SEMUSEFLAG_IN_USE	EQU	0 
SEMUSEFLAG_NOT_IN_USE	EQU	1 
;  DevHelp_SemHandle   
EVENT_MOUSEHOTKEY	EQU	0 
EVENT_CTRLBREAK	EQU	1 
EVENT_CTRLC	EQU	2 
EVENT_CTRLNUMLOCK	EQU	3 
EVENT_CTRLPRTSC	EQU	4 
EVENT_SHIFTPRTSC	EQU	5 
EVENT_KBDHOTKEY	EQU	6 
EVENT_KBDREBOOT	EQU	7 
; 
;* Timer Management
; 
;* Real Mode Helpers
; 
;* Monitors
;  DevHelp_Register   
CHAIN_AT_TOP	EQU	0 
CHAIN_AT_BOTTOM	EQU	1 
; 
;* OS/2 2.x Only
;*
;* Note: These DeviceHelps are not available on OS/2 1.x systems
;*
; 
;* 32-Bit Memory Management
;  DevHelp_VMLock   
VMDHL_NOBLOCK	EQU	0001H 
VMDHL_CONTIGUOUS	EQU	0002H 
VMDHL_16M	EQU	0004H 
VMDHL_WRITE	EQU	0008H 
VMDHL_LONG	EQU	0010H 
VMDHL_VERIFY	EQU	0020H 
;  DevHelp_VMAlloc                                                   /*@V76282*/
                                                                     ; @V76282  
VMDHA_16M	EQU	0001H                                ; @V76282  
VMDHA_FIXED	EQU	0002H                                ; @V76282  
VMDHA_SWAP	EQU	0004H                                ; @V76282  
VMDHA_CONTIG	EQU	0008H                                ; @V76282  
VMDHA_PHYS	EQU	0010H                                ; @V76282  
VMDHA_PROCESS	EQU	0020H                                ; @V76282  
VMDHA_SGSCONT	EQU	0040H                                ; @V76282  
VMDHA_RESERVE	EQU	0100H                                ; @V76282  
VMDHA_USEHIGHMEM	EQU	0800H                                ; @V76282  
;  DevHelp_VMGlobalToProcess                                         /*@V76282*/
                                                                     ; @V76282  
VMDHGP_WRITE	EQU	0001H                                ; @V76282  
VMDHGP_SELMAP	EQU	0002H                                ; @V76282  
VMDHGP_SGSCONTROL	EQU	0004H                                ; @V76282  
VMDHGP_4MEG	EQU	0008H                                ; @V76282  
;  DevHelp_VMProcessToGlobal                                         /*@V76282*/
                                                                     ; @V76282  
VMDHPG_READONLY	EQU	0000H                                ; @V76282  
VMDHPG_WRITE	EQU	0001H                                ; @V76282  
;  DevHelp_GetDescInfo                                               /*@V76282*/
                                                                     ; @V76282  

SELDESCINFO	STRUC
                                                                     ; @V76282  
DHSI_Type	DB	?                                                     ; @V76282  
DHSI_Granularity	DB	?                                              ; @V76282  
DHSI_BaseAddr	DD	?                                                 ; @V76282  
DHSI_Limit	DD	?                                                    ; @V76282  
                                                                     ; @V76282  
SELDESCINFO	ENDS
                                                                     ; @V76282  
                                                                     ; @V76282  

GATEDESCINFO	STRUC
                                                                     ; @V76282  
DHGI_Type	DB	?                                                     ; @V76282  
DHGI_ParmCount	DB	?                                                ; @V76282  
DHGI_Selector	DB	SIZE SEL DUP (?)                                                 ; @V76282  
DHGI_Reserved_1	DW	?                                               ; @V76282  
DHGI_Offset	DD	?                                                   ; @V76282  
                                                                     ; @V76282  
GATEDESCINFO	ENDS
                                                                     ; @V76282  
;  DevHelp_PageListToGDTSelector                                     /*@V76282*/
;  DevHelp_PageListToGDTSel                                          /*@V76282*/
                                                                     ; @V76282  
GDTSEL_R3CODE	EQU	0000H                                ; @V76282  
GDTSEL_R3DATA	EQU	0001H                                ; @V76282  
GDTSEL_R2CODE	EQU	0003H                                ; @V76282  
GDTSEL_R2DATA	EQU	0004H                                ; @V76282  
GDTSEL_R0CODE	EQU	0005H                                ; @V76282  
GDTSEL_R0DATA	EQU	0006H                                ; @V76282  
;  GDTSEL_ADDR32 may be OR'd with above defines                      /*@V76282*/
GDTSEL_ADDR32	EQU	0080H                                ; @V76282  
;  DevHelp_VMSetMem                                                  /*@V76282*/
                                                                     ; @V76282  
VMDHS_DECOMMIT	EQU	0001H                                ; @V76282  
VMDHS_RESIDENT	EQU	0002H                                ; @V76282  
VMDHS_SWAP	EQU	0004H                                ; @V76282  
;  PageList structure                                                /*@V76282*/
                                                                     ; @V76282  

PAGELIST	STRUC
                                                                     ; @V76282  
DHPL_PhysAddr	DD	?                                                 ; @V76282  
DHPL_Size	DD	?                                                     ; @V76282  
                                                                     ; @V76282  
PAGELIST	ENDS
                                                                     ; @V76282  
; 
;* 32-Bit Context Hooks
; 
;* Adapter Device Drivers / Device Managers
;  DevHelp_RegisterDeviceClass                                       /*@V74979*/
                                                                     ; @V74979  
DEVICECLASS_ADDDM	EQU	1                                     ; @V74979  
DEVICECLASS_MOUSE	EQU	2                                     ; @V74979  
; 
;* Miscellaneous
                                                                     ; @V76282  
                                                                     ; @V76282  
;  DevHelp_DynamicAPI                                                /*@V76282*/
DYNAPI_CALLGATE16	EQU	0001H    ;  16:16 CallGate           /*@V76282*/
DYNAPI_CALLGATE32	EQU	0000H    ;   0:32 CallGate           /*@V76282*/
                                                                     ; @V76282  
DYNAPI_ROUTINE16	EQU	0002H    ;  16:16 Routine Addr       /*@V76282*/
DYNAPI_ROUTINE32	EQU	0000H    ;   0:32 Routine Addr       /*@V76282*/
;  File Systems DevHelp routines for BaseDev Init Time Only      

FOPEN	STRUC
DHFSO_FileName	DB	SIZE PSZ DUP (?)
DHFSO_FileSize	DD	?
FOPEN	ENDS

FCLOSE	STRUC
DHFSC_Reserved	DW	?
FCLOSE	ENDS

FREAD	STRUC
DHFSR_Buffer	DB	SIZE PBYTE DUP (?)
DHFSR_ReadSize	DD	?
FREAD	ENDS

FREADAT	STRUC
DHFSA_Buffer	DB	SIZE PBYTE DUP (?)
DHFSA_ReadSize	DD	?
DHFSA_StartPosition	DD	?
FREADAT	ENDS
?UnionSize = 0
if size FOPEN gt ?UnionSize
	?UnionSize = size FOPEN
endif
FileOpen	equ	(byte ptr 0)

if size FCLOSE gt ?UnionSize
	?UnionSize = size FCLOSE
endif
FileClose	equ	(byte ptr 0)

if size FREAD gt ?UnionSize
	?UnionSize = size FREAD
endif
FileRead	equ	(byte ptr 0)

if size FREADAT gt ?UnionSize
	?UnionSize = size FREADAT
endif
FileReadAt	equ	(byte ptr 0)


FILEIOOP	STRUC
	DB	?UnionSize dup(?)
FILEIOOP	ENDS


DDFileIo	STRUC
DHFSIO_Length	DW	?
DHFSIO_Data	DB	SIZE FILEIOOP DUP (?)
DDFileIo	ENDS
; 
;* DevHlp Error Codes  (from ABERROR.INC)
MSG_MEMORY_ALLOCATION_FAILED	EQU	00H 
ERROR_LID_ALREADY_OWNED	EQU	01H 
ERROR_LID_DOES_NOT_EXIST	EQU	02H 
ERROR_ABIOS_NOT_PRESENT	EQU	03H 
ERROR_NOT_YOUR_LID	EQU	04H 
ERROR_INVALID_ENTRY_POINT	EQU	05H 
