;**************************************************************************
;*
;* SOURCE FILE NAME =  SCB.H
;*
;* DESCRIPTIVE NAME =  ADD/DM include file
;*                     Subsystem Control Block (SCB) structures
;*
;* Copyright : COPYRIGHT IBM CORPORATION, 1991, 1992
;*             LICENSED MATERIAL - PROGRAM PROPERTY OF IBM
;*             REFER TO COPYRIGHT INSTRUCTION FORM#G120-2083
;*             RESTRICTED MATERIALS OF IBM
;*             IBM CONFIDENTIAL
;*
;* VERSION = V2.0
;*
;* DATE
;*
;* DESCRIPTION :
;*
;* Purpose:      Defines the Subsystem Control Block used by the Option
;*               Device Drivers written to SCSI.SYS (OS2SCSI.DMD)
;*               interfaces.
;*
;*               See ..\OEMSPEC\SCSISPEC.ASC for further information.
;*
;* FUNCTIONS  :
;*
;*
;*
;* NOTES
;*
;*
;* STRUCTURES
;*
;* EXTERNAL REFERENCES
;*
;*
;*
;* EXTERNAL FUNCTIONS
;*
;* CHANGE ACTIVITY =
;*   DATE      FLAG        APAR   CHANGE DESCRIPTION
;*   --------  ----------  -----  --------------------------------------
;*
;*   06/11/92       RAB           PTR CP20 B736829
;*                                Modify default SCB Enable words
;*                                By R. Berkoff
;*
;***************************************************************************  
; 
;* Subsystem Control Block Structure

PSCBHDR struc
        dd      ?
PSCBHDR ends

; 
;*  ABIOS SCB Header

SCBHDR	STRUC
ABSCB_Reserved_1	DW	?             ;  Reserved                          
ABSCB_pNextSCBHdr	DD	?            ;  Logical ptr to next SCB header    
ABSCB_Reserved_2	DW	2 DUP (?)          ;  Reserved                          
ABSCB_pTSB	DD	?                   ;  Logical ptr to TSB                
ABSCB_Reserved_3	DW	1 DUP (?)          ;  reserved                          
SCBHDR	ENDS
; 
;*  SCSI Subsystem Control Block (SCB)

SCB	STRUC
SCB_Cmd	DW	?                    ;  SCB Command Code                     
SCB_Enable	DW	?                 ;  SCB Enable Word                      
SCB_LBA	DD	?                    ;  Logical Block Addr                   
SCB_ppXferBuf	DD	?              ;  Physical pointer to transfer buf     
                                        ;   or scatter/gather list              
SCB_XferBufLen	DD	?             ;  Length of transfer or addr list      
SCB_ppTSB	DD	?                  ;  Physical pointer to TSB              
SCB_ppNxtSCB	DD	?               ;  Physical ptr to next SCB on chain    
?UnionSize = 0
SCB_BlockCnt	DW	?       ;  Block count                          
SCB_BlockSize	DW	?      ;  Block length                         
SCB	ENDS
; 
;* SCB Variant for FORMAT

SCBFORMAT	STRUC
SCBFMT_Cmd	DW	?                    ;  SCB Command Code                     
SCBFMT_Enable	DW	?                 ;  SCB Enable Word                      
SCBFMT_ModBits	DW	?                ;  Modifier Bits  FD/CL                 
SCBFMT_Interleave	DW	?             ;  Interleave Factor                    
SCBFMT_ppXferBuf	DD	?              ;  Physical pointer to transfer buf     
                                        ;   or scatter/gather list              
SCBFMT_XferBufLen	DD	?             ;  Length of transfer or addr list      
SCBFMT_ppTSB	DD	?                  ;  Physical pointer to TSB              
SCBFMT_ppNxtSCB	DD	?               ;  Physical ptr to next SCB on chain    
?UnionSize = 0
SCBFMT_BlockCnt	DW	?       ;  Block count                          
SCBFMT_BlockSize	DW	?      ;  Block length                         
SCBFORMAT	ENDS
; 
;* SCBCmd codes
SCBREAD	EQU	1C01H           ;  SCB Read                             
SCBWRITE	EQU	1C02H           ;  SCB Write                            
SCBREADV	EQU	1C03H           ;  SCB Read with Verify                 
SCBWRITEV	EQU	1C04H           ;  SCB Write with Verify                
SCBCMDSTATUS	EQU	1C07H           ;  SCB Get Command Complete Status      
SCBCMDSENSE	EQU	1C08H           ;  SCB Req SCSI Sense Command           
SCBDEVICECAP	EQU	1C09H           ;  SCB Read Device Capacity             
SCBQUERYPOSINFO	EQU	1C0AH           ;  SCB Query Adapter POS Info           
SCBDEVICEINQ	EQU	1C0BH           ;  SCB Device Inquiry                   
SCBREASSIGNBLK	EQU	1C18H           ;  SCB Reassign Block                   
SCBMAXLBA	EQU	1C1AH           ;  SCB Specify Maximum LBA              
SCBSENDOTHER	EQU	241FH           ;  SCB Send Other SCSI Command          
SCBPREFETCH	EQU	1C31H           ;  SCB Prefetch                         
SCBFORMATUNIT	EQU	1C16H           ;  SCB Format Unit                      
; 
;* SCBCmd Bit Flags
SCBCfND	EQU	0080H           ;  SCB Suppress Target Disconnect       
SCBCfNS	EQU	0040H           ;  SCB Suppress Sync. Negotiation       
; 
;* SCBEnable word bit flag definitions
SCBEfRD	EQU	8000H        ;  I/O Control:  1=read,                   
                                     ;                0=write                   
SCBEfES	EQU	4000H        ;  Return TSB:   1=only on error,          
                                     ;                0=always                  
SCBEfRE	EQU	2000H        ;  Retry enable: 1=enable retries,         
                                     ;                0=disable retries         
SCBEfPT	EQU	1000H        ;  SGList: 1=SCBXferBuf is SGList          
                                     ;          0=SCBXferBuf is SCBXferBuf      
SCBEfSS	EQU	0400H        ;  Suppress except: 1=suppress error       
                                     ;                   0=don't suppress       
SCBEfBB	EQU	0200H        ;  Bypass cache: 1=bypass cache            
                                     ;                0=don't bypass            
SCBEfCC	EQU	0001H        ;  Chain Condition: 1=chain                
                                     ;                   0=no chain             
; 
;* SCBEnable default word definitions
SCBEWREAD	EQU	SCBEfRD+SCBEfES+SCBEfRE+SCBEfPT
SCBEWWRITE	EQU	SCBEfES+SCBEfRE+SCBEfPT
SCBEWREADV	EQU	SCBEfRD+SCBEfES+SCBEfRE+SCBEfBB
SCBEWWRITEV	EQU	SCBEfES+SCBEfRE+SCBEfPT
SCBEWCMDSTATUS	EQU	SCBEfRD+SCBEfES+SCBEfRE+SCBEfBB
SCBEWCMDSENSE	EQU	SCBEfRD+SCBEfES+SCBEfRE+SCBEfSS+SCBEfBB    ;  @V2.0R-B00  
SCBEWDEVICECAP	EQU	SCBEfRD+SCBEfES+SCBEfRE+SCBEfBB
SCBEWDEVICEINQ	EQU	SCBEfRD+SCBEfES+SCBEfRE+SCBEfSS+SCBEfBB
SCBEWREASSIGNBLK	EQU	SCBEfES+SCBEfRE+SCBEfBB
SCBEWMAXLBA	EQU	0 
SCBEWSENDOTHER	EQU	SCBEfES+SCBEfRE+SCBEfPT+SCBEfBB+SCBEfSS   ;  @V2.0R-B00  
SCBEWPREFETCH	EQU	SCBEfRD+SCBEfES+SCBEfRE+SCBEfBB
SCBEWFORMATUNIT	EQU	SCBEfES+SCBEfRE+SCBEfBB
SCBEWDEFAULT	EQU	SCBEfES+SCBEfRE
; 
;* Termination Status Block

TSB	STRUC
TSB_Status	DW	?              ;  Ending status                           
TSB_Retries	DW	?             ;  Retry count                             
TSB_ResidCnt	DD	?            ;  Residual byte count                     
TSB_ppResidBuf	DD	?          ;  Residual physical buffer addr           
TSB_StatusLen	DW	?           ;  Additional status length                
TSB_SCSIStatus	DB	?          ;  SCSI Status                             
TSB_CmdStatus	DB	?           ;  Command status                          
TSB_DevError	DB	?            ;  Device error code                       
TSB_CmdError	DB	?            ;  Command error code                      
TSB_DiagMod	DW	?             ;  Diagnostic error modifier               
TSB_CacheInfo	DW	?           ;  Cache info word                         
TSB_ppLastSCB	DD	?           ;  Physical ptr to last SCB processed      
TSB	ENDS
; 
;* Bit Flags for TSB->Status
TSBSfNOERR	EQU	0001H        ;  No error has occured                    
TSBSfSHORT	EQU	0002H        ;  Short length record encountered         
TSBSfSPECCHK	EQU	0010H        ;  SCB Specification Check                 
TSBSfLONG	EQU	0020H        ;  Long record encountered                 
TSBSfHALT	EQU	0040H        ;  SCB Chain halted                        
TSBSfINTREQ	EQU	0080H        ;  SCB Interrupt Requested                 
TSBSfRESIDOK	EQU	0100H        ;  Resid buffer data there                 
TSBSfSTATF	EQU	0200H        ;  Up to word 0F of TSB valid              
TSBSfSTATX	EQU	0300H        ;  Extended TSB Format Stored              
TSBSfOVERRUN	EQU	0400H        ;  Device Overrun                          
TSBSfNOTINIT	EQU	0800H        ;  Device not initialized                  
TSBSfEXCEPT	EQU	1000H        ;  Major exception has occurred            
TSBSfCHDIR	EQU	2000H        ;  Chain direction bit                     
TSBSfSUSPEND	EQU	4000H        ;  SCB Suspended                           
TSBSfXSTAT	EQU	8000H        ;  Extend end status word                  
; 
;* Bit Flags for TSB->Retries
RC_ADAPTER_RETRY	EQU	8000H     ;  Adapter retry invoked         
RC_SYSTEN_CHECK_RETRY	EQU	0020H     ;  System interface check retry  
; 
;* Equates for TSB->SCSIStatus
;  Refer to SCSI.H (SCSI_STAT_*)
; 
;* Equates for TSB->CmdStatus
CS_COMP_W_SUCCESS	EQU	01H     ;  Cmd comp w/ success           
CS_COMP_W_SUCCESS_RETRY	EQU	05H     ;  Cmd comp w/ suc. after retry  
CS_ADAPTER_HW_ERROR	EQU	07H     ;  Adapter hardware failure      
CS_IMM_CMD_COMP	EQU	0AH     ;  Immediate cmd comp            
CS_COMP_W_FAILURE	EQU	0CH     ;  Cmp comp w/ failure           
CS_COMMAND_ERROR	EQU	0EH     ;  Commnad error                 
CS_SW_SEQUENCE_ERROR	EQU	0FH     ;  Software sequencing error     
; 
;* Equates for TSB->CmdError
CE_NO_ERROR	EQU	00H     ;  No error                      
CE_INVALID_PARM	EQU	01H     ;  Invalid parameter in SCB      
CE_CMD_NOT_SUPPORT	EQU	03H     ;  Commnad not supported         
CE_CMD_ABORT	EQU	04H     ;  Commnad aborted ( by system   
CE_FORMAT_REJECT	EQU	07H     ;  Format reject-sequence error  
CE_ASSIGN_REJECT_1	EQU	08H     ;  Assign rej -cmd in progress   
CE_ASSIGN_REJECT_2	EQU	09H     ;           -dev already assign  
CE_CMD_REJECT	EQU	0AH     ;  Cmd rej -dev not assigned     
CE_MAX_LBA_EXCEED	EQU	0BH     ;  Max LBA exceeded              
CE_16_CARDSLOTADDR_EXCEED	EQU	0CH     ;  16bits card slot addr range   
                                              ;                    exceeded    
CE_INVALID_DEV_FOR_CMD	EQU	13H     ;  Invalid device for commnan    
CE_ADAPTER_HW_ERR	EQU	20H     ;  Adapter hardware error        
CE_GLOBAL_CMD_TIMEOUT	EQU	21H     ;  Global commnad timeout        
CE_DMA_ERR	EQU	22H     ;  DMA error                     
CE_ADAP_BUFF_DEFECTIVE	EQU	23H     ;  Adapter buffer defective      
CE_CMD_ABORT_BY_ADAPT	EQU	24H     ;  Command aborted by adapter    
CE_ADAPT_DETECT_ERR	EQU	80H     ;  Adapt microprocessor          
                                              ;          defected error        
; 
;* Equates for TSB->DevError
DE_NO_ERROR	EQU	00H     ;  No error                      
DE_SCSIBUS_RESET	EQU	01H     ;  SCSI bus reset occurred       
DE_SCSI_INTF_FAULT	EQU	02H     ;  SCSI interface fault          
DE_SCSI_SELCT_TIMEOUT	EQU	00H     ;  SCSI selection timeout        
                                              ;       ( device not available)  
DE_SCSIBUS_FREE	EQU	00H     ;  Unexpected SCSI bus free      
DE_INV_SCSI_PHASE_SEQ	EQU	00H     ;  Invalid SCSI phase sequence   
DE_SHORT_LEN_RECORD	EQU	00H     ;  Short length record           
; 
;* Bit Flags for TSB->CacheInfo
CI_CACHE_ENABLED	EQU	0800H     ;  Cache enabled                 
CI_CACHE_RETRY_OCCUR	EQU	0400H     ;  Cache retry occurred          
CI_TOTAL_WRITE_HIT	EQU	0200H     ;  Total write hit               
CI_TOTAL_READ_HIt	EQU	0100H     ;  Total read hit                
